library IEEE,mylib;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use mylib.vtx_pack.all;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;

entity evt_bldr is
    port (
-- reset input, should be connected to the inverted LOCK output of dll
        reset: in STD_LOGIC;
        reset_err: in STD_LOGIC;
-- system clock input
        clk: in STD_LOGIC;
        version : in std_logic_vector(15 downto 0);
-- ttcrx event numbers
	 	  ovfl_warning : out std_logic;
	 	  evn_fifo_full : out std_logic;
	 	  event_number_avl : in std_logic;
        event_number : in std_logic_vector(71 downto 0);
-- LRB data spying
		  lrb_en : in std_logic_vector(17 downto 0);
-- lrbx_we signals lrbx_data are valid
		  src_sel : IN std_logic_vector(1 downto 0);
		  lrb1_we : IN std_logic;
		  lrb2_we : IN std_logic;
-- bit 35 is truncate flag, bit 34 reset flag
-- bit 33 and 32 used to indicate header and trailer of the block
		  lrb1_data : IN std_logic_vector(35 downto 0);
		  lrb2_data : IN std_logic_vector(35 downto 0);
		  lrb_data : OUT std_logic_vector(33 downto 0);
-- w_addr(11 downto 5) buffer number write pointer
-- w_addr(4 downto 0) HTR number
		  w_addr : OUT std_logic_vector(11 downto 0);
-- monitored event buffer number
		  mon_wp : IN std_logic_vector(6 downto 0);
-- download data write
        addr: in STD_LOGIC_VECTOR (13 downto 0);
        w_data: in STD_LOGIC_VECTOR (31 downto 0);
        r_data: out STD_LOGIC_VECTOR (31 downto 0);
        run: in STD_LOGIC;
        n_strobe: in STD_LOGIC;
        is_high: in STD_LOGIC;
        rd_evn_pntr : in std_logic;
        rd_evn_buf : in std_logic;
-- synchronization timeout setting
        wr_timeout: in STD_LOGIC;
        rd_timeout: in STD_LOGIC;
-- source ID
        wr_src_id: in STD_LOGIC;
        rd_src_id: in STD_LOGIC;
-- read HTR error counter
        rd_status_cntr : in std_logic;
        wr_stateLUT : in std_logic;
        rd_stateLUT : in std_logic;
        wr_mmLUT : in std_logic;
        rd_mmLUT : in std_logic;
		  HTR_ow : OUT std_logic;
		  HTR_bz : OUT std_logic;
        err_state: out STD_LOGIC_VECTOR (1 downto 0);
-- counter inputs
        rd_evn_mm : in std_logic;
        rd_counters : in std_logic;
        inc_l1ac : in std_logic;
        inc_bcnterr : in std_logic;
        inc_oc : in std_logic;
        inc_serr : in std_logic;
        inc_derr : in std_logic;
        inc_rate_OFW : in std_logic;
        inc_mon_cntr : in std_logic;
        inc_slink_wc : in std_logic;
        inc_slink_bc : in std_logic;
-- LED display signals
		  htr_status : out std_logic_vector(14 downto 0);
		  htr_error : out std_logic_vector(14 downto 0);
-- memory readout
-- evt_bldr calculates from htr_buf_gnt when the data will be
-- available.
        htr_buf_gnt: in STD_LOGIC;
        htr_buf_req: out STD_LOGIC;
-- bit(14 downto 8) buffer number address
-- bit(7 downto 3) lrb_id
-- bit(2 downto 0) block address
        htr_buf_addr: out STD_LOGIC_VECTOR (14 downto 0);
-- htr_buf_length is 32-bit word counts
        htr_buf_length: out STD_LOGIC_VECTOR (5 downto 0);
        htr_buf_re: out STD_LOGIC;
        htr_buf_data: in STD_LOGIC_VECTOR (31 downto 0);
        htr_buf_full: out STD_LOGIC_VECTOR (14 downto 0);
-- TTCrx states
		  TTS : IN std_logic_vector(3 downto 0);
-- slink data 
-- fifo full
        slink_full: in STD_LOGIC;
-- data in write enable
        slink_we: out STD_LOGIC;
-- slink_data(35) is Calib_trig flag
-- slink_data(34) is slink_ctrl, slink_data(33) is header, slink_data(32) is trailer
        slink_data: out STD_LOGIC_VECTOR (35 downto 0);
-- debugging control
		  rd_debug: in std_logic
    );
end evt_bldr;

architecture my_arch of evt_bldr is
signal strobe: std_logic := '0';
signal slink_we_i: std_logic := '0';
signal event_data: std_logic_vector(34 downto 0) := (others => '0');
signal event_data_dl: std_logic_vector(15 downto 0) := (others => '0');
signal d_crc_data: std_logic_vector(31 downto 0) := (others => '0');
signal crc_data: std_logic_vector(31 downto 0) := (others => '0');
signal crc_data_a: std_logic_vector(3 downto 0) := (others => '0');
signal crc: std_logic_vector(15 downto 0) := (others => '0');
signal calib_trig: std_logic := '0';
signal init_crc: std_logic := '0';
signal sel_crc: std_logic := '0';
signal first_word: std_logic := '0';
signal high_word: std_logic := '0';
signal wr_high: std_logic := '0';
signal wr_low: std_logic := '0';
signal we_crc: std_logic := '0';
signal lrb_data_i: std_logic_vector(35 downto 0) := (others => '0');
signal rst_cycle: std_logic := '0';
signal rst_cycle_dl: std_logic := '0';
signal rst_done: std_logic := '0';
signal we_wp: std_logic := '0';
signal lrb_trailer: std_logic := '0';
signal we_rp: std_logic := '0';
signal wp_plus_one: std_logic_vector(6 downto 0) := (others => '0');
signal dpo_wp: std_logic_vector(6 downto 0) := (others => '0');
signal dpo_wp_q: std_logic_vector(6 downto 0) := (others => '0');
signal wp: std_logic_vector(6 downto 0) := (others => '0');
signal wp_addr: std_logic_vector(3 downto 0) := (others => '0');
signal dpo_rp: std_logic_vector(6 downto 0) := (others => '0');
signal dpo_rp_q: std_logic_vector(6 downto 0) := (others => '0');
signal rp_plus_one: std_logic_vector(6 downto 0) := (others => '0');
signal rp: std_logic_vector(6 downto 0) := (others => '0');
signal rp_addr: std_logic_vector(3 downto 0) := (others => '0');
signal last_lrb: std_logic := '0';
signal lrb1_id: std_logic_vector(3 downto 0) := x"0";
signal lrb2_id: std_logic_vector(3 downto 0) := x"0";
signal htr_en: std_logic_vector(14 downto 0) := (others => '0');
signal htr_buf_cnt_w: std_logic_vector(6 downto 0) := (others => '0');
signal htr_buf_cnt_r: std_logic_vector(6 downto 0) := (others => '0');
signal htr_buf_cnt_w_q: std_logic_vector(1 downto 0) := (others => '0');
signal htr_buf_not_full: std_logic := '0';
signal htr_dav: std_logic := '0';
signal sel_header: std_logic := '0';
signal ec_header_fifo_wa_h: std_logic := '0';
signal wr_padding: std_logic := '0';
signal start_header: std_logic := '0';
signal oc_match: std_logic := '0';
signal bc_match: std_logic := '0';
signal evn_match: std_logic := '0';
signal ec_bc_evn_match: std_logic := '0';
signal bc_evn_match: std_logic_vector(3 downto 0) := (others => '0');
signal lrb_smry: std_logic_vector(31 downto 0) := (others => '0');
signal summary: std_logic_vector(31 downto 0) := (others => '0');
signal htr_header: std_logic_vector(34 downto 0) := (others => '0');
signal header: std_logic_vector(35 downto 0) := (others => '0');
signal daq_header: std_logic_vector(35 downto 0) := (others => '0');
signal daq_header1: std_logic_vector(35 downto 0) := (others => '0');
signal daq_header2: std_logic_vector(35 downto 0) := (others => '0');
signal dcc_header: std_logic_vector(35 downto 0) := (others => '0');
signal wr_summary: std_logic := '0';
signal EPV: std_logic_vector(14 downto 0) := (others => '0');
signal daq_trailer: std_logic_vector(34 downto 0) := (others => '0');
signal evt_length: std_logic_vector(12 downto 0) := (others => '0');
signal header_in: std_logic_vector(35 downto 0) := (others => '0');
signal header_out: std_logic_vector(35 downto 0) := (others => '0');
signal header_fifo_wa: std_logic_vector(8 downto 0) := (others => '0');
signal header_fifo_ra: std_logic_vector(8 downto 0) := (others => '0');
constant twentyfour: std_logic_vector(12 downto 0) := "0000000011000";
signal empty_header_fifo: std_logic := '0';
signal full_header_fifo: std_logic := '0';
signal trailer_sent: std_logic := '0';
signal wr_header: std_logic := '0';
signal ec_header_fifo_ra: std_logic := '0';
signal header_sent: std_logic := '0';
signal end_of_htr: std_logic := '0';
signal htr_sent: std_logic := '0';
signal chk_htr_req_cnt: std_logic := '0';
signal wr_htr_req_cnt: std_logic := '0';
signal htr_req_cnt: std_logic_vector(6 downto 0) := (others => '0');
signal htr_req_cnt_que: std_logic_vector(6 downto 0) := (others => '0');
signal not_htr_rd_cnt: std_logic_vector(6 downto 0) := (others => '0');
signal ec_htr_rd_cnt: std_logic := '0';
signal start_trailer: std_logic := '0';
signal fake_lrb: std_logic := '0';
signal wr_htr: std_logic := '0';
signal wr_tail: std_logic := '0';
signal wr_tail_dl: std_logic := '0';
signal sel_head: std_logic := '0';
signal wr_head: std_logic := '0';
signal ec_rp_addr: std_logic := '0';
signal clear: std_logic := '0';
signal check_next: std_logic := '0';
signal skip: std_logic := '0';
signal missing: std_logic := '0';
signal start_read: std_logic := '0';
signal evn_seq: std_logic := '0';
signal we_last_evn: std_logic := '0';
signal last_evn: std_logic_vector(7 downto 0) := (others => '0');
signal evn_diff: std_logic_vector(23 downto 0) := (others => '0');
signal lrb_match: std_logic_vector(11 downto 0) := (others => '0');
signal lrb_match_q: std_logic_vector(2 downto 0) := (others => '0');
signal matching: std_logic_vector(3 downto 0) := (others => '0');
signal en_scan: std_logic := '0';
signal htr_scan: std_logic := '0';
signal stop_htr_scan: std_logic := '0';
signal two_htr_buf: std_logic := '0';
signal one_evn: std_logic := '0';
signal htr_time_out: std_logic := '0';
signal ld_evn_wait: std_logic := '0';
signal evn_wait_ovfl: std_logic := '0';
signal evn_timeout: std_logic := '0';
signal d_evn_wait: std_logic_vector(9 downto 0) := (others => '0');
signal ld_htr_wait: std_logic := '0';
signal htr_wait_ovfl: std_logic := '0';
signal d_htr_wait: std_logic_vector(14 downto 0) := (others => '0');
signal d_to_cnt: std_logic_vector(1 downto 0) := (others => '0');
signal to_cnt: std_logic_vector(1 downto 0) := (others => '0');
signal evn_delta: std_logic_vector(7 downto 0) := (others => '0');
signal new_rp: std_logic := '0';
signal new_rp_dl: std_logic := '0';
signal wr_htr_req_que: std_logic := '0';
signal en_htr_buf_re: std_logic := '0';
signal read_htr_buf: std_logic := '0';
signal odd_wc: std_logic := '0';
signal ec_htr_rd_burst: std_logic := '0';
signal d_htr_rd_burst: std_logic_vector(6 downto 0) := (others => '0');
signal htr_rd_burst: std_logic_vector(6 downto 0) := (others => '0');
signal htr_rd_length: std_logic_vector(9 downto 0) := (others => '0');
signal htr_req_que_in: std_logic_vector(31 downto 0) := (others => '0');
signal htr_req_queue: std_logic_vector(31 downto 0) := (others => '0');
signal htr_req_fifo_wa: std_logic_vector(8 downto 0) := (others => '0');
signal htr_req_fifo_ra: std_logic_vector(8 downto 0) := (others => '0');
signal htr_req_que_wc: std_logic_vector(8 downto 0) := (others => '0');
signal htr_req_que_wc_up: std_logic := '0';
signal empty_htr_req_que: std_logic := '0';
signal htr_rd_que_in: std_logic_vector(5 downto 0) := (others => '0');
signal htr_rd_queue: std_logic_vector(5 downto 0) := (others => '0');
signal wr_htr_rd_que: std_logic := '0';
signal empty_htr_rd_que: std_logic := '0';
signal full_htr_rd_que: std_logic := '0';
signal blk_num: std_logic_vector(2 downto 0) := (others => '0');
signal last_htr_blk: std_logic := '0';
signal evn_avl: std_logic := '0';
signal wr_evn: std_logic := '0';
signal ovfl_warning_i : std_logic := '0';
signal evn_fifo_full_i : std_logic := '0';
signal evn_fifo_wa: std_logic_vector(9 downto 0) := (others => '0');
signal evn_fifo_rap: std_logic_vector(9 downto 0) := (others => '0');
signal evn_fifo_ra: std_logic_vector(9 downto 0) := (others => '0');
signal evn_wc: std_logic_vector(7 downto 0) := (others => '0');
signal ld_fake_evn: std_logic := '0';
signal fake_evn: std_logic_vector(71 downto 0) := (others => '0');
signal evn_in: std_logic_vector(71 downto 0) := (others => '0');
signal evn_out: std_logic_vector(71 downto 0) := (others => '0');
signal evn_data: std_logic_vector(31 downto 0) := (others => '0');
signal lrb_wa: std_logic_vector(10 downto 0) := (others => '0');
signal evn_ra: std_logic_vector(10 downto 0) := (others => '0');
signal smry_ra: std_logic_vector(10 downto 0) := (others => '0');
signal lrb_evn: std_logic_vector(41 downto 0) := (others => '0');
signal source_id: std_logic_vector(31 downto 0) := (others => '0');
signal timeout: std_logic_vector(31 downto 0) := x"00000000";
signal NoL1ac: std_logic := '0';
signal NoWaitNext: std_logic := '0';
signal NoSkip: std_logic := '0';
signal NoMiss: std_logic := '0';
signal EnFakeL1ac: std_logic := '0';
signal EnHTRtimeout: std_logic := '0';
signal lrb_total_wc: std_logic_vector(9 downto 0) := (others => '0');
signal lrb_wc_que: std_logic_vector(19 downto 0) := (others => '0');
signal wr_lrb_total_wc: std_logic := '0';
signal rd_lrb_wc_que: std_logic := '0';
signal empty_lrb_wc_que: std_logic := '0';
signal start_scan: std_logic := '0';
signal rd_htr_que: std_logic := '0';
signal htr_bc_done: std_logic := '0';
signal d_htr_cntr_addr: std_logic_vector(5 downto 0) := (others => '0');
signal htr_cntr_addr: std_logic_vector(5 downto 0) := (others => '0');
signal htr_addr: std_logic_vector(4 downto 0) := (others => '0');
signal offset: std_logic_vector(4 downto 0) := (others => '0');
signal htr_wc: std_logic_vector(8 downto 0) := (others => '0');
signal EvN_mm: std_logic := '0';
signal BcN_mm: std_logic := '0';
signal CT_EvN_mm: std_logic := '0';
signal CT_BcN_mm: std_logic := '0';
signal L1_EvN_mm: std_logic := '0';
signal L1_BcN_mm: std_logic := '0';
signal en_wr_cntr_que_a: std_logic := '0';
signal DCC_err: std_logic_vector(11 downto 0) := (others => '0');
-- 32 bit counters
signal evt_bldr_bc: std_logic_vector(1 downto 0) := (others => '0');
signal slink_bc: std_logic_vector(1 downto 0) := (others => '0');
signal mon_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal l1ac_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal CT_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal CT_EvN_mmc: std_logic_vector(1 downto 0) := (others => '0');
signal CT_BcN_mmc: std_logic_vector(1 downto 0) := (others => '0');
signal L1_EvN_mmc: std_logic_vector(1 downto 0) := (others => '0');
signal L1_BcN_mmc: std_logic_vector(1 downto 0) := (others => '0');
signal bcnterr_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal rate_OFW_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal ttc_state: std_logic_vector(1 downto 0) := (others => '0');
signal ld_evt_bldr_bc: std_logic := '0';
signal ld_slink_bc: std_logic := '0';
signal ld_mon_cntr: std_logic := '0';
signal ld_l1ac_cntr: std_logic := '0';
signal ld_CT_cntr: std_logic := '0';
signal ld_CT_EvN_mmc: std_logic := '0';
signal ld_CT_BcN_mmc: std_logic := '0';
signal ld_L1_EvN_mmc: std_logic := '0';
signal ld_L1_BcN_mmc: std_logic := '0';
signal ld_bcnterr_cntr: std_logic := '0';
signal ld_rate_OFW_cntr: std_logic := '0';
signal rst_counterb: std_logic := '0';
signal counterb_inc: std_logic_vector(31 downto 0) := (others => '0');
signal counterb_addr: std_logic_vector(3 downto 0) := (others => '0');
signal counterb_in: std_logic_vector(31 downto 0) := (others => '0');
signal counterb_out: std_logic_vector(31 downto 0) := (others => '0');
signal counterb: std_logic_vector(31 downto 0) := (others => '0');
signal counterb_q: std_logic_vector(31 downto 0) := (others => '0');
-- 64 bit counters
signal evt_bldr_wc: std_logic_vector(7 downto 0) := (others => '0');
signal slink_wc: std_logic_vector(6 downto 0) := (others => '0');
signal serr_cntr: std_logic_vector(5 downto 0) := (others => '0');
signal derr_cntr: std_logic_vector(5 downto 0) := (others => '0');
signal ready_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal busy_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal ovfl_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal sync_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal run_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal ld_evt_bldr_wc: std_logic := '0';
signal ld_slink_wc: std_logic := '0';
signal ld_serr_cntr: std_logic := '0';
signal ld_derr_cntr: std_logic := '0';
signal ld_ready_cntr: std_logic := '0';
signal ld_busy_cntr: std_logic := '0';
signal ld_ovfl_cntr: std_logic := '0';
signal ld_sync_cntr: std_logic := '0';
signal ld_run_cntr: std_logic := '0';
signal d_evt_bldr_wc: std_logic_vector(7 downto 0) := (others => '0');
signal ec_evt_bldr_wc: std_logic := '0';
signal mux2_a: std_logic_vector(7 downto 0) := (others => '0');
signal mux2_b: std_logic_vector(7 downto 0) := (others => '0');
signal mux2_c: std_logic_vector(7 downto 0) := (others => '0');
signal mux2_d: std_logic_vector(7 downto 0) := (others => '0');
signal mux4_a: std_logic_vector(7 downto 0) := (others => '0');
signal mux4_b: std_logic_vector(7 downto 0) := (others => '0');
signal scan_cntr_que_a: std_logic := '0';
signal scan_cntr_que_a_dl: std_logic := '0';
signal scan_cntr_que_a_dl2: std_logic := '0';
signal wr_cntr_que_a: std_logic := '0';
signal rd_cntr_que_a: std_logic := '0';
signal empty_cntr_que_a: std_logic := '0';
signal sel_cntr_que_a: std_logic := '0';
signal sel_cntr_que_b: std_logic := '0';
signal sel_cntr_que_c: std_logic := '0';
signal cntr_que_a_in: std_logic_vector(11 downto 0) := (others => '0');
signal cntr_que_a: std_logic_vector(11 downto 0) := (others => '0');
signal scan_cntr: std_logic_vector(3 downto 0) := (others => '0');
signal scan_cntr_b: std_logic_vector(3 downto 0) := (others => '0');
signal wr_cntr_que_bp: std_logic := '0';
signal wr_cntr_que_b: std_logic := '0';
signal rd_cntr_que_b: std_logic := '0';
signal empty_cntr_que_b: std_logic := '0';
signal counter_cycle: std_logic_vector(1 downto 0) := (others => '0');
signal wr_counter: std_logic := '0';
signal rst_counter_n: std_logic := '0';
signal en_counter_out: std_logic := '0';
signal ec_counter_wap: std_logic := '0';
signal cntr_que_b_in: std_logic_vector(8 downto 0) := (others => '0');
signal cntr_que_b: std_logic_vector(8 downto 0) := (others => '0');
signal cntr_que_c: std_logic_vector(16 downto 0) := (others => '0');
signal rd_cntr_que_c: std_logic := '0';
signal empty_cntr_que_c: std_logic := '0';
signal d_counter_wap: std_logic_vector(6 downto 0) := (others => '0');
signal counter_wap: std_logic_vector(6 downto 0) := (others => '0');
signal counter_wa: std_logic_vector(8 downto 0) := (others => '0');
signal counter_ra: std_logic_vector(8 downto 0) := (others => '0');
signal counters: std_logic_vector(31 downto 0) := (others => '0');
signal increment: std_logic_vector(63 downto 0) := (others => '0');
signal counter_in: std_logic_vector(63 downto 0) := (others => '0');
signal counter_out: std_logic_vector(63 downto 0) := (others => '0');
signal counter_rb: std_logic_vector(63 downto 0) := (others => '0');
signal counter_high: std_logic_vector(31 downto 0) := (others => '0');
signal old_cntr_addr: std_logic_vector(6 downto 0) := (others => '0');
signal evn_mm_pc_in : std_logic_vector(31 downto 0) := (others => '0');
signal evn_mm_pc_out : std_logic_vector(31 downto 0) := (others => '0');
signal next_evn_mm_pc_aa : std_logic_vector(5 downto 0) := (others => '0');
signal evn_mm_pc_aa : std_logic_vector(10 downto 0) := (others => '0');
signal evn_mm_pc_ab : std_logic_vector(8 downto 0) := (others => '0');
signal inc_evn_mm_cntr : std_logic_vector(31 downto 0) := (others => '0');
signal evn_mm_cntr_in : std_logic_vector(31 downto 0) := (others => '0');
signal evn_mm_cntr_out : std_logic_vector(31 downto 0) := (others => '0');
signal evn_mm_cntr_aa : std_logic_vector(8 downto 0) := (others => '0');
signal evn_mm_cntr_ab : std_logic_vector(8 downto 0) := (others => '0');
signal evn_mm_buf_a : std_logic_vector(8 downto 0) := (others => '0');
signal evn_mm_buf : std_logic_vector(31 downto 0) := (others => '0');
signal clr_evn_mm_pc : std_logic := '0';
signal wr_evn_mm_cntr : std_logic := '0';
signal rst_mm_cntr : std_logic := '0';
signal HTR_state: std_logic_vector(1 downto 0) := (others => '0');
signal mmLUT : std_logic_vector(7 downto 0) := (others => '0');
signal mmLUT_rb : std_logic_vector(7 downto 0) := (others => '0');
signal mmLUT_spo : std_logic_vector(7 downto 0) := (others => '0');
signal we_mmLUT: std_logic := '0';
signal mmLUT_scan : std_logic_vector(3 downto 0) := (others => '0');
signal CT_err_cntr : std_logic_vector(7 downto 0) := (others => '0');
signal L1_err_cntr : std_logic_vector(7 downto 0) := (others => '0');
signal err_cntr : std_logic_vector(3 downto 0) := (others => '0');
signal threshold : std_logic_vector(3 downto 0) := (others => '0');
signal stateToGo : std_logic_vector(1 downto 0) := (others => '0');
signal bcnt_err: std_logic := '0';
signal bcnt_err_cntr : std_logic_vector(3 downto 0) := (others => '0');
signal faked_evn: std_logic := '0';
signal saved_TTS : std_logic_vector(3 downto 0) := (others => '0');
------------------------------------------------------
-- debugging
signal evn_to: std_logic := '0';
signal debug: std_logic_vector(95 downto 0) := (others => '0');
------------------------------------------------------
begin
evn_fifo_full <= evn_fifo_full_i;
ovfl_warning <= ovfl_warning_i;
lrb_data <= lrb_data_i(33 downto 0);
process(clk)
begin
  if(clk'event and clk = '1')then
    strobe <= n_strobe;
  end if;
end process;
-- htr buffer full/empty signal
i_htr_buf_cnt_w: adsuN
   generic map( N => 7)
   port map(
         a => wp,
         b => dpo_rp,
         ci => '1',
         add => '0',
         s => htr_buf_cnt_w);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   htr_buf_cnt_w_q <= "00";
	   htr_buf_not_full <= '0';
	 else
	   htr_buf_cnt_w_q <= htr_buf_cnt_w(6 downto 5);
	   htr_buf_not_full <= not htr_buf_cnt_r(5);
    end if;
    if(rst_done = '1')then
	   htr_buf_full <= (others => '0');
    elsif(reset = '1')then
	   htr_buf_full <= (others => '1');
	 elsif(we_wp = '1' and htr_buf_cnt_w_q = "11")then
	   case wp_addr is
		  when "0000" => htr_buf_full(0) <= '1';
		  when "0001" => htr_buf_full(1) <= '1';
		  when "0010" => htr_buf_full(2) <= '1';
		  when "0011" => htr_buf_full(3) <= '1';
		  when "0100" => htr_buf_full(4) <= '1';
		  when "0101" => htr_buf_full(5) <= '1';
		  when "0110" => htr_buf_full(6) <= '1';
		  when "0111" => htr_buf_full(7) <= '1';
		  when "1000" => htr_buf_full(8) <= '1';
		  when "1001" => htr_buf_full(9) <= '1';
		  when "1010" => htr_buf_full(10) <= '1';
		  when "1011" => htr_buf_full(11) <= '1';
		  when "1100" => htr_buf_full(12) <= '1';
		  when "1101" => htr_buf_full(13) <= '1';
		  when "1110" => htr_buf_full(14) <= '1';
		  when others => null;
		end case;
	 elsif(strobe = '1' and wr_htr_req_que = '1' and htr_buf_not_full = '1')then
	   case rp_addr is
		  when "0000" => htr_buf_full(0) <= '0';
		  when "0001" => htr_buf_full(1) <= '0';
		  when "0010" => htr_buf_full(2) <= '0';
		  when "0011" => htr_buf_full(3) <= '0';
		  when "0100" => htr_buf_full(4) <= '0';
		  when "0101" => htr_buf_full(5) <= '0';
		  when "0110" => htr_buf_full(6) <= '0';
		  when "0111" => htr_buf_full(7) <= '0';
		  when "1000" => htr_buf_full(8) <= '0';
		  when "1001" => htr_buf_full(9) <= '0';
		  when "1010" => htr_buf_full(10) <= '0';
		  when "1011" => htr_buf_full(11) <= '0';
		  when "1100" => htr_buf_full(12) <= '0';
		  when "1101" => htr_buf_full(13) <= '0';
		  when "1110" => htr_buf_full(14) <= '0';
		  when others => null;
		end case;
    end if;
  end if;
end process;
w_addr(4 downto 0) <= '0' & wp_addr;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(src_sel(1) = '0')then
	   w_addr(11 downto 5) <= mon_wp;
	 else
	   w_addr(11 downto 5) <= wp;
    end if;
    if(src_sel(0) = '0')then
	   lrb_data_i <= lrb1_data;
	 else
	   lrb_data_i <= lrb2_data;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(rst_cycle = '1')then
	   wp_addr <= rp_addr;
    elsif(src_sel(0) = '0')then
	   wp_addr <= lrb1_id;
	 else
	   wp_addr <= lrb2_id;
    end if;
  end if;
end process;
-- buffer pointers
i_wp: ram16XNd
    generic map( N => 7)
    port map(
        clk => clk,
        we => we_wp, 
        d => wp_plus_one,
        a => wp_addr,
        dpra => rp_addr,
        spo => wp,
        dpo => dpo_wp);			
i_wp_plus_one: l_cntr
  generic map(N => 7,ld_plus_one => true)
  port map(
    d => wp,
    load => '1',
    reset => rst_cycle,
    set => '0',
    ec => '1',
    en_cnt => '1',
    clk => clk,
	 q => wp_plus_one);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   we_wp <= '0';
		lrb_trailer <= '0';
	 else
      we_wp <= rst_cycle or lrb_trailer; 
      lrb_trailer <= (lrb1_we and lrb1_data(32)) or 
               (lrb2_we and lrb2_data(32));
    end if;
  end if;
end process;
-- read pointer
i_rp: ram16XNd
    generic map( N => 7)
    port map(
        clk => clk,
        we => we_rp, 
        d => rp_plus_one,
        a => rp_addr,
        dpra => wp_addr,
        spo => rp,
        dpo => dpo_rp);
i_rp_plus_one: l_cntr
  generic map(N => 7,ld_plus_one => true)
  port map(
    d => rp,
    load => '1',
    reset => rst_cycle,
    set => '0',
    ec => strobe,
    en_cnt => '1',
    clk => clk,
	 q => rp_plus_one);
we_rp <= strobe and (rst_cycle or skip or (wr_htr_req_que and last_htr_blk));
i_htr_buf_cnt_r: adsuN
   generic map( N => 7)
   port map(
         a => dpo_wp_q,
         b => rp,
         ci => '1',
         add => '0',
         s => htr_buf_cnt_r);
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(strobe = '1')then
	   dpo_wp_q <= dpo_wp;
	   dpo_rp_q <= dpo_rp;
    end if;
  end if;
end process;
-- reset starts a cycle to clear all pointers
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   rst_cycle <= '1';
	   rst_cycle_dl <= '0';
	 elsif(strobe = '1')then
	   rst_cycle <= rst_cycle and not last_lrb;
	   rst_cycle_dl <= rst_cycle;
    end if;
  end if;
end process;
-- read pointer address
i_rp_addr: l_cntr
  generic map(N => 4)
  port map(
    d => x"0",
    load => last_lrb,
    reset => reset,
    set => '0',
    ec => ec_rp_addr,
    en_cnt => '1',
    clk => clk,
    q => rp_addr);
last_lrb <= '1' when rp_addr(3 downto 1) = "111" else '0';
ec_rp_addr <= strobe and (rst_cycle or (wr_htr_req_que and last_htr_blk) or fake_lrb or missing);
-- remember the current LRB id
process(clk)
begin
  if(clk'event and clk = '1')then
    if(lrb1_data(33) = '1')then
	   case lrb1_data(11 downto 8) is
		  when "0100" => lrb1_id <= "0010";
		  when "0101" => lrb1_id <= "0001";
		  when "0110" => lrb1_id <= "0000";
		  when "1000" => lrb1_id <= "0101";
		  when "1001" => lrb1_id <= "0100";
		  when "1010" => lrb1_id <= "0011";
		  when others => lrb1_id <= "1111";
		end case;
    end if;
    if(lrb2_data(33) = '1')then
	   case lrb2_data(11 downto 8) is
		  when "0000" => lrb2_id <= "1000";
		  when "0001" => lrb2_id <= "0111";
		  when "0010" => lrb2_id <= "0110";
		  when "0100" => lrb2_id <= "1011";
		  when "0101" => lrb2_id <= "1010";
		  when "0110" => lrb2_id <= "1001";
		  when "1000" => lrb2_id <= "1110";
		  when "1001" => lrb2_id <= "1101";
		  when "1010" => lrb2_id <= "1100";
		  when others => lrb2_id <= "1111";
		end case;
    end if;
  end if;
end process;
-- slink output control
-- to include calculated crc into daq trailer
i_slink_we: delayN generic map(N => 6) port map(d => slink_we_i, ce => '1', c => clk, q => slink_we);
i_calib_trig: delayN generic map(N => 6) port map(d => calib_trig, ce => '1', c => clk, q => slink_data(35));
i_slink_data_h: delayt_N generic map(t => 6, N => 3) port map(d => event_data(34 downto 32), ce => '1', clk => clk, q => slink_data(34 downto 32));
i_slink_data_l: delayt_N generic map(t => 6, N => 16) port map(d => event_data(15 downto 0), ce => '1', clk => clk, q => slink_data(15 downto 0));
i_event_data_dl: delayt_N generic map(t => 5, N => 16) port map(d => event_data(31 downto 16), ce => '1', clk => clk, q => event_data_dl);
i_sel_crc: delayN generic map(N => 7) port map(d => start_trailer, ce => '1', c => clk, q => sel_crc);
-- this is used to swap the order of the high and low 32-bit words
i_crc_data: srle_N 
    generic map(N=>32)
	 port map(
	      d => event_data(31 downto 0),
			a => crc_data_a,
			ce => slink_we_i,
			clk => clk,
			q => d_crc_data);
crc_data_a(3 downto 2) <= "00";
i_init_crc: delayN generic map(N => 3) port map(d => first_word, ce => '1', c => clk, q => init_crc);
process(clk)
begin
  if(clk'event and clk = '1')then
    crc_data <= d_crc_data;
	 first_word <= sel_head and not wr_head;
	 high_word <= (high_word xor slink_we_i) and not first_word;
    crc_data_a(1) <= slink_we_i and not high_word;
    crc_data_a(0) <= not slink_we_i and not high_word;
-- two clocks after high word written to swapper, writes both words to crc16D32
	 wr_high <= high_word and slink_we_i;
	 wr_low <= wr_high;
	 we_crc <= wr_high or wr_low;
    if(sel_crc = '1')then
      slink_data(31 downto 16) <= crc;
	 else
      slink_data(31 downto 16) <= event_data_dl;
	 end if;
  end if;
end process;
i_crc: crc16D32
      port map(
		    clk => clk,
			 init_crc => init_crc,
			 we_crc => we_crc,
			 d => crc_data,
			 crc => crc);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   start_trailer <= '0';
	   wr_tail_dl <= '0';
	   htr_dav <= '0';
	   wr_padding <= '0';
	   sel_head <= '0';
	   wr_head <= '0';
	   wr_htr <= '0';
	   wr_tail <= '0';
	   slink_we_i <= '0';
	 else
	   start_trailer <= htr_sent or (chk_htr_req_cnt and end_of_htr and not read_htr_buf);
	   wr_tail_dl <= wr_tail;
	   htr_dav <= not htr_rd_burst(6);
-- add padding if htr data are not 64-bit aligned
	   wr_padding <= htr_sent and (evt_length(0) xor wr_htr);
	   sel_head <= ec_header_fifo_ra;
	   wr_head <= sel_head;
-- discard last htr readout word if word count is odd
	   wr_htr <= htr_dav and not(htr_rd_burst(6) and odd_wc);
		wr_tail <= start_trailer or (wr_tail and not wr_tail_dl);
	   slink_we_i <= wr_htr or wr_padding or wr_head or wr_tail;
    end if;
  end if;
end process;
i_evt_length: l_cntr
  generic map(N => 13)
  port map(
    d => twentyfour,
    load => header_sent,
    reset => reset,
    set => '0',
    ec => slink_we_i,
    en_cnt => '1',
    clk => clk,
    q => evt_length);
-- output multiplexer
process(clk)
begin
  if(clk'event and clk = '1')then
    if(wr_padding = '1')then
-- fill with zeros
	   event_data <= (others => '0');
    elsif(wr_tail = '1')then
-- daq trailer
	   event_data <= daq_trailer;
	 else
-- daq header, dcc header, htr summary and htr data
	   event_data <= htr_header;
    end if;
    if(sel_head = '1')then
-- daq header, dcc header and htr header
	   htr_header <= header_out(34 downto 0);
	 else
-- htr data
	   htr_header <= "000" & htr_buf_data;
    end if;
-- check calib_trig
	 if(first_word = '1')then
	   calib_trig <= header_out(35);
	 end if;
    if(wr_tail = '1')then
-- daq trailer high word
	   daq_trailer <= "101" & x"a0000" & evt_length(12 downto 1);
	 else
-- daq trailer low word	with crc forced to zero
	   daq_trailer <= "100" & x"0000" & source_id(19 downto 12) & TTS & x"0";
    end if;
-- DAQ header, DCC header and HTR summary
    if(strobe = '1')then
	   if(sel_header = '1')then
	     header_in <= header;
	   elsif(fake_lrb = '1' or missing = '1')then
        header_in <= x"00000" & lrb_smry(15) & "000" & x"000";
	   else
        header_in <= x"0" & lrb_smry(31 downto 14) & (bc_evn_match(2) and bc_evn_match(1)) & bc_evn_match(0) & lrb_smry(11 downto 0);
      end if;
	   if(header_fifo_wa(2 downto 1) = "01")then
        header <= dcc_header;
-- high word here
        dcc_header <= x"000" & version(7 downto 0) & x"0" & DCC_err;
	   else
	     header <= daq_header;
-- low word here
        dcc_header <= "0000000" & EPV & "00" & saved_TTS & x"02";
      end if;
	   if(sel_header = '1')then
        daq_header <= daq_header2;
	   else
	     daq_header <= daq_header1;
      end if;
		case evn_out(71 downto 69) is
		  when "001" => saved_TTS <= x"1";
		  when "010" => saved_TTS <= x"2";
		  when "011" => saved_TTS <= x"4";
		  when "100" => saved_TTS <= x"8";
		  when "101" => saved_TTS <= x"c";
		  when "110" => saved_TTS <= x"f";
		  when others => null;
		end case;
	 end if;
  end if;
end process;
lrb_smry <= summary(31 downto 16) & htr_en(0) & summary(14 downto 0); 
daq_header1 <= x"45" & source_id(23 downto 20) & evn_out(23 downto 0) when start_header = '1'
	        else evn_out(68) & "110" & evn_out(35 downto 24) & source_id(11 downto 0) & x"08";
--              high word										  low word
daq_header2 <= x"4" & evn_out(63 downto 36) & x"0" when wr_header = '0' else x"40000000" & evn_out(67 downto 64);
i_header_fifo: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => header_in(31 downto 0),
  			  DIPA => header_in(35 downto 32),
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => strobe,
  			  ENB => '1',
  			  WEA => wr_header,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => header_fifo_wa,
			  ADDRB => header_fifo_ra,
			  DOB => header_out(31 downto 0),
			  DOPB => header_out(35 downto 32)
			  );
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
		header_fifo_wa(8 downto 5) <= "0000";
	 elsif(strobe = '1')then
		header_fifo_wa(8 downto 5) <= header_fifo_wa(8 downto 5) + ec_header_fifo_wa_h;
	 end if;
	 if(strobe = '1')then
	   if(sel_header = '1')then
		  header_fifo_wa(4 downto 0) <= header_fifo_wa(4 downto 0) + 1;
		elsif(start_header = '1')then
		  header_fifo_wa(4 downto 0) <= "11111";
		else
		  header_fifo_wa(4 downto 0) <= ('0' & rp_addr) + "00110";
		end if;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(strobe = '1')then
	   wr_summary <= start_read or fake_lrb or missing;
	   if(wr_summary = '1')then
		  EPV(14) <= header_in(15) and header_in(14) and header_in(13) and header_in(12) and not header_in(11);
	     EPV(13 downto 0) <= EPV(14 downto 1);
		end if;
		if(ec_header_fifo_wa_h = '1')then
		  htr_status <= EPV;
		  htr_error <= not EPV and lrb_en(17 downto 3);
		  EvN_mm <= '0';
		  BcN_mm <= '0';
		  CT_EvN_mm <= evn_out(68) and EvN_mm;
		  CT_BcN_mm <= evn_out(68) and BcN_mm;
		  L1_EvN_mm <= not evn_out(68) and EvN_mm;
		  L1_BcN_mm <= not evn_out(68) and BcN_mm;
		else
		  htr_status <= (others => '0');
		  htr_error <= (others => '0');
		  if(wr_summary = '1' and header_in(15) = '1' and (header_in(14) = '0' or header_in(12) = '0'))then
		    EvN_mm <= '1';
		  end if;
		  if(wr_summary = '1' and header_in(15) = '1' and header_in(14 downto 13) /= "11")then
		    BcN_mm <= '1';
		  end if;
		  CT_EvN_mm <= '0';
		  CT_BcN_mm <= '0';
		  L1_EvN_mm <= '0';
		  L1_BcN_mm <= '0';
		end if;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   sel_header <= '0';
	   wr_header <= '0';
	   ec_header_fifo_wa_h <= '0';
	 elsif(strobe = '1')then
	   sel_header <= start_header or (sel_header and not (not header_fifo_wa(3) and header_fifo_wa(2)));
	   wr_header <= sel_header or start_read or fake_lrb or missing;
	   ec_header_fifo_wa_h <= sel_header and not header_fifo_wa(3) and header_fifo_wa(2);
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   start_header <= '0';
	   fake_lrb <= '0';
	 elsif(strobe = '1')then
	   start_header <= stop_htr_scan;
	   fake_lrb <= not fake_lrb and not new_rp and not htr_en(0) and
		            ((en_scan and not last_lrb) or htr_scan);
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   header_fifo_ra(8 downto 5) <= "0000";
	 elsif(header_sent = '1')then
	   header_fifo_ra(8 downto 5) <= header_fifo_ra(8 downto 5) + 1;
    end if;
    if(reset = '1')then
	   header_fifo_ra(4 downto 0) <= "00000";
	 elsif(ec_header_fifo_ra = '1')then
	   if(header_sent = '1')then
	     header_fifo_ra(4 downto 0) <= "00000";
		else
	     header_fifo_ra(4 downto 0) <= header_fifo_ra(4 downto 0) + 1;
		end if;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   trailer_sent <= '1';
	   ec_header_fifo_ra <= '0';
	   header_sent <= '0';
	 else
	   trailer_sent <= start_trailer or (trailer_sent and not ec_header_fifo_ra);
	   ec_header_fifo_ra <= (trailer_sent and not empty_header_fifo) or (ec_header_fifo_ra and not header_sent);
	   header_sent <= header_fifo_ra(4) and header_fifo_ra(2) and header_fifo_ra(1) and not header_fifo_ra(0);
    end if;
  end if;
end process;
i_htr_req_cnt: l_cntr
  generic map(N => 7)
  port map(
    d => "0000000",
    load => ec_header_fifo_wa_h,
    reset => reset,
    set => '0',
    ec => strobe,
    en_cnt => wr_htr_req_que,
    clk => clk,
    q => htr_req_cnt);
wr_htr_req_cnt <= strobe and ec_header_fifo_wa_h;
i_htr_req_cnt_que : srl_fiforeg
generic map (N => 7,full_cnt => 15)
  port map(
	din => htr_req_cnt,
	write => wr_htr_req_cnt,
	read => header_sent, 
	clk => clk,
	rst => reset,
	dout => htr_req_cnt_que,
	empty => empty_header_fifo,
	full => full_header_fifo);
not_htr_rd_cnt <= not htr_req_cnt_que;
i_htr_rd_cnt: l_cntr
  generic map(N => 7)
  port map(
    d => not_htr_rd_cnt,
    load => header_sent,
    reset => reset,
    set => '0',
    ec => ec_htr_rd_cnt,
    en_cnt => '1',
    clk => clk,
    carry => end_of_htr);
ec_htr_rd_cnt <= en_htr_buf_re or header_sent;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   chk_htr_req_cnt <= '0';
	 else
	   chk_htr_req_cnt <= ec_htr_rd_cnt;
    end if;
  end if;
end process;
-- LRB enable
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(strobe = '1')then
	   if(en_scan = '0' and htr_scan = '0')then
-- mip1 not used
		  htr_en <= lrb_en(17 downto 3);
		elsif(ec_rp_addr = '1')then
		  htr_en <= htr_en(0) & htr_en(14 downto 1);
		end if;
    end if;
  end if;
end process;
oc_match <= '1' when evn_out(40 downto 36) = lrb_evn(40 downto 36);
i_bc_match: compCN
  generic map(N => 12)
  port map(
    a => evn_out(35 downto 24),
    b => lrb_evn(35 downto 24),
    en => '1',
    equal => bc_match);
i_evn_diff: adsuN
   generic map( N => 24)
   port map(
         a => evn_out(23 downto 0),
         b => lrb_evn(23 downto 0),
         ci => '1',
         add => '0',
         s => evn_diff);
evn_match <= '1' when evn_diff = x"000000" else '0';
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(strobe = '1' and ec_bc_evn_match = '1')then
		bc_evn_match <= evn_out(68) & oc_match & bc_match & evn_match;
    end if;
	 if(strobe = '1')then
	   ec_bc_evn_match <= not two_htr_buf or new_rp or en_scan;
	 end if;
  end if;
end process;
--ec_bc_evn_match <= strobe and (not two_htr_buf or new_rp_dl);
-- HTR buffer readout control
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   en_htr_buf_re <= '0';
	   read_htr_buf <= '0';
	   htr_sent <= '0';
	 else
	   en_htr_buf_re <= not en_htr_buf_re and read_htr_buf and not empty_htr_rd_que and not slink_full and htr_rd_burst(6);
	   read_htr_buf <= (chk_htr_req_cnt and not end_of_htr) or (read_htr_buf and not chk_htr_req_cnt);
	   htr_sent <= not read_htr_buf and htr_rd_burst(6) and htr_dav;
    end if;
-- remember if read length is odd
	 if(en_htr_buf_re = '1')then
	   odd_wc <= not htr_rd_queue(0);
	 end if;
  end if;
end process;
i_htr_rd_burst: l_cntr
  generic map(N => 7)
  port map(
    d => d_htr_rd_burst,
    load => htr_rd_burst(6),
    reset => '0',
    set => reset,
    ec => ec_htr_rd_burst,
    en_cnt => '1',
    clk => clk,
    q => htr_rd_burst);
-- always read even number of words. Discard the last word if length is odd
d_htr_rd_burst <= '0' & not htr_rd_queue(5 downto 1) & '0';
ec_htr_rd_burst <= not htr_rd_burst(6) or en_htr_buf_re;
htr_buf_re <= not htr_rd_burst(6);
-- htr_rd_que is smaller than the ddr read fifo
htr_buf_req <= not empty_htr_req_que and not full_htr_rd_que;
htr_buf_addr <= htr_req_queue(20 downto 6);
htr_buf_length <= htr_req_queue(5 downto 0);
htr_req_que_in(31 downto 6) <= "00000000000" & rp & '0' & rp_addr & blk_num;
i_htr_req_fifo: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => htr_req_que_in(31 downto 0),
  			  DIPA => x"0",
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => strobe,
  			  ENB => '1',
  			  WEA => wr_htr_req_que,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => htr_req_fifo_wa,
			  ADDRB => htr_req_fifo_ra,
			  DOB => htr_req_queue(31 downto 0)
			  );
i_htr_req_fifo_wa: l_cntr
  generic map(N => 9)
  port map(
    d => "000000000",
    load => '0',
    reset => reset,
    set => '0',
    ec => strobe,
    en_cnt => wr_htr_req_que,
    clk => clk,
    q => htr_req_fifo_wa);
i_htr_req_fifo_ra: l_cntr
  generic map(N => 9)
  port map(
    d => "000000000",
    load => '0',
    reset => reset,
    set => '0',
    ec => htr_buf_gnt,
    en_cnt => '1',
    clk => clk,
    q => htr_req_fifo_ra);
i_htr_req_que_wc: updown_cntr
  generic map(N => 9)
  port map(
    rst => reset,
    set => '0',
    ce => '1',
    up => htr_req_que_wc_up,
    down => htr_buf_gnt,
    clk => clk,
	 q => htr_req_que_wc);
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(reset = '1')then
	   htr_req_que_wc_up <= '0';
	 else
	   htr_req_que_wc_up <= wr_htr_req_que and strobe;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(reset = '1')then
	   empty_htr_req_que <= '1';
    elsif(htr_req_que_wc = "0000000000")then
	   empty_htr_req_que <= '1';
	 else
	   empty_htr_req_que <= '0';
	 end if;
    if(htr_buf_gnt = '1')then
	   htr_rd_que_in <= htr_req_queue(5 downto 0);
    end if;
  end if;
end process;
I_wr_htr_rd_que: delayN generic map(N => 5) port map(d => htr_buf_gnt, ce => '1', c => clk, q => wr_htr_rd_que);
i_htr_rd_queue : srl_fiforeg
generic map (N => 6,full_cnt => 15)
  port map(
	din => htr_rd_que_in,
	write => wr_htr_rd_que,
	read => en_htr_buf_re, 
	clk => clk,
	rst => reset,
	dout => htr_rd_queue,
	empty => empty_htr_rd_que,
	full => full_htr_rd_que);
i_blk_num: l_cntr
  generic map(N => 3)
  port map(
    d => "000",
    load => start_read,
    reset => '0',
    set => '0',
    ec => strobe,
    en_cnt => wr_htr_req_que,
    clk => clk,
	 q => blk_num);
last_htr_blk <= '1' when blk_num = htr_rd_length(8 downto 6) else '0';
htr_req_que_in(5 downto 0) <= "111111" when last_htr_blk = '0' else
	   								htr_rd_length(5 downto 0);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   wr_htr_req_que <= '0';
	 elsif(strobe = '1')then
	   wr_htr_req_que <= start_read or (wr_htr_req_que and not last_htr_blk);
    end if;
  end if;
end process;
-- HTR readout length
i_htr_rd_length: l_d_cntr
  generic map(N => 10,ld_minus_one => true)
  port map(
    d => summary(9 downto 0),
    load => '1',
    reset => '0',
    set => '0',
    ec => strobe,
    en_cnt => '0',
    clk => clk,
	 q => htr_rd_length);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
		evn_timeout <= evn_wait_ovfl;
	 end if;
  end if;
end process;
i_fake_evn: l_cntr
  generic map(N => 24,ld_plus_one => true)
  port map(
    d => event_number(23 downto 0),
    load => ld_fake_evn,
    reset => reset,
    set => '0',
    ec => strobe,
    en_cnt => evn_timeout,
    clk => clk,
	 q => fake_evn(23 downto 0));
ld_fake_evn <= event_number_avl and not NoL1ac;
fake_evn(71 downto 24) <= x"e00000000000";
evn_in <= fake_evn when evn_timeout = '1' else event_number;
-- TTCrx event number fifo
g_evn_fifo: for i in 0 to 3 generate
  i_evn_fifo: RAMB16_S18_S18
	  generic map(SIM_COLLISION_CHECK => "NONE")
    port map(DIA => evn_in(i*18+15 downto i*18),
  			  DIPA => evn_in(i*18+17 downto i*18+16),
  			  DIB => x"0000",
  			  DIPB => "00",
  			  ENA => strobe,
  			  ENB => strobe,
  			  WEA => wr_evn,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => evn_fifo_wa,
			  ADDRB => evn_fifo_rap,
			  DOB => evn_out(i*18+15 downto i*18),
			  DOPB => evn_out(i*18+17 downto i*18+16)
			  );
end generate;
evn_fifo_rap <= evn_fifo_ra when run = '1' else addr(11 downto 2);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   evn_fifo_wa <= (others => '0');
	   evn_fifo_ra <= (others => '0');
    elsif(strobe = '1')then
	   if(wr_evn = '1')then
		  evn_fifo_wa <= evn_fifo_wa - 1;
		end if;
	   if(ec_header_fifo_wa_h = '1')then
		  evn_fifo_ra <= evn_fifo_ra - 1;
		end if;
	 end if;
  end if;
end process;
i_evn_wc: updown_cntr
  generic map(N => 8)
  port map(
    rst => reset,
    set => '0',
    ce => strobe,
    up => wr_evn,
    down => ec_header_fifo_wa_h,
    clk => clk,
	 q => evn_wc);
evn_avl <= '0' when evn_wc = x"00" else '1';
one_evn <= '1' when evn_wc = x"01" else '0';
wr_evn <= '0' when evn_fifo_full_i = '1' else evn_timeout when NoL1ac = '1' else
			 event_number_avl;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   new_rp <= '1';
	   new_rp_dl <= '0';
		ovfl_warning_i <= '0';
		evn_fifo_full_i <= '0';
	 elsif(strobe = '1')then
		new_rp <= fake_lrb or missing or skip or (wr_htr_req_que and last_htr_blk);
		new_rp_dl <= new_rp;
-- when reached 0x60, throttle lv1. Return only after go below 0x40
		if(evn_wc(7 downto 6) = "00")then
		  ovfl_warning_i <= '0';
      elsif(evn_wc(5) = '1')then
		  ovfl_warning_i <= '1';
		end if;
		if(evn_wc(7 downto 4) = x"f")then
		  evn_fifo_full_i <= '1';
		else
		  evn_fifo_full_i <= '0';
		end if;
    end if;
  end if;
end process;
two_htr_buf <= '0' when htr_buf_cnt_r(6 downto 1) = "000000" else '1';
i_matching: RAMB16_S4
     generic map(
-- algorithm
      INIT_00 => x"1188118888888888118811888888888820202020202020202020202020202020",
      INIT_01 => x"1188118888888888118811888888888888444444444444448844444444444444",
      INIT_02 => x"8880808080808080882010808080808088808080808080808820108080808080",
      INIT_03 => x"8880808080808080882010808080808088808080808080808820108080808080",
      INIT_04 => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_05 => x"8888888888888888888888888888888888444444444444448844444444444444",
      INIT_06 => x"8880808080808080882010808080808088808080808080808820108080808080",
      INIT_07 => x"8880808080808080882010808080808088808080808080808820108080808080",
      INIT_08 => x"1188118888888888118811888888888820202020202020202020202020202020",
      INIT_09 => x"1188118888888888118811888888888888444444444444448844444444444444",
      INIT_0a => x"8880808080808080888010808080808088808080808080808880108080808080",
      INIT_0b => x"8880808080808080888010808080808088808080808080808880108080808080",
      INIT_0c => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_0d => x"8888888888888888888888888888888888444444444444448844444444444444",
      INIT_0e => x"8880808080808080888010808080808088808080808080808880108080808080",
      INIT_0f => x"8880808080808080888010808080808088808080808080808880108080808080",
      INIT_10 => x"1188118811221188118811881122118820202020202020202020202020202020",
      INIT_11 => x"1188118811221188118811881122118888444444884444448844444488444444",
      INIT_12 => x"8880808088808080882010808820108088808080888080808820108088201080",
      INIT_13 => x"8880808088808080882010808820108088808080888080808820108088201080",
      INIT_14 => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_15 => x"8888888888888888888888888888888888444444884444448844444488444444",
      INIT_16 => x"8880808088808080882010808820108088808080888080808820108088201080",
      INIT_17 => x"8880808088808080882010808820108088808080888080808820108088201080",
      INIT_18 => x"1188118811881188118811881188118820202020202020202020202020202020",
      INIT_19 => x"1188118811881188118811881188118888444444884444448844444488444444",
      INIT_1a => x"8880808088808080888010808880108088808080888080808880108088801080",
      INIT_1b => x"8880808088808080888010808880108088808080888080808880108088801080",
      INIT_1c => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_1d => x"8888888888888888888888888888888888444444884444448844444488444444",
      INIT_1e => x"8880808088808080888010808880108088808080888080808880108088801080",
      INIT_1f => x"8880808088808080888010808880108088808080888080808880108088801080",
      INIT_20 => x"1188118888888888118811888888888820202020202020202020202020202020",
      INIT_21 => x"1188118888888888118811888888888888444444444444448844444444444444",
      INIT_22 => x"8888888888888888882211888888888888888888888888888822118888888888",
      INIT_23 => x"8888888888888888882211888888888888888888888888888822118888888888",
      INIT_24 => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_25 => x"8888888888888888888888888888888888444444444444448844444444444444",
      INIT_26 => x"8888888888888888882211888888888888888888888888888822118888888888",
      INIT_27 => x"8888888888888888882211888888888888888888888888888822118888888888",
      INIT_28 => x"1188118888888888118811888888888820202020202020202020202020202020",
      INIT_29 => x"1188118888888888118811888888888888444444444444448844444444444444",
      INIT_2a => x"8888888888888888888811888888888888888888888888888888118888888888",
      INIT_2b => x"8888888888888888888811888888888888888888888888888888118888888888",
      INIT_2c => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_2d => x"8888888888888888888888888888888888444444444444448844444444444444",
      INIT_2e => x"8888888888888888888811888888888888888888888888888888118888888888",
      INIT_2f => x"8888888888888888888811888888888888888888888888888888118888888888",
      INIT_30 => x"1188118811221188118811881122118820202020202020202020202020202020",
      INIT_31 => x"1188118811221188118811881122118888444444884444448844444488444444",
      INIT_32 => x"8888888888888888882211888822118888888888888888888822118888221188",
      INIT_33 => x"8888888888888888882211888822118888888888888888888822118888221188",
      INIT_34 => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_35 => x"8888888888888888888888888888888888444444884444448844444488444444",
      INIT_36 => x"8888888888888888882211888822118888888888888888888822118888221188",
      INIT_37 => x"8888888888888888882211888822118888888888888888888822118888221188",
      INIT_38 => x"1188118811881188118811881188118820202020202020202020202020202020",
      INIT_39 => x"1188118811881188118811881188118888444444884444448844444488444444",
      INIT_3a => x"8888888888888888888811888888118888888888888888888888118888881188",
      INIT_3b => x"8888888888888888888811888888118888888888888888888888118888881188",
      INIT_3c => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_3d => x"8888888888888888888888888888888888444444884444448844444488444444",
      INIT_3e => x"8888888888888888888811888888118888888888888888888888118888881188",
      INIT_3f => x"8888888888888888888811888888118888888888888888888888118888881188")
  port map(DI => x"0",
  			  EN => strobe,
  			  WE => '0',
  			  SSR => clear,
			  CLK => clk,
			  ADDR => lrb_match,
			  DO => matching(3 downto 0)
			  );
clear <= reset or not htr_scan or wr_htr_req_que or new_rp or matching(3) or matching(1) or matching(0);
-- if NoWaitNext set, do not wait for next HTR event
lrb_match(11) <= NoWaitNext when matching(2) = '0' else lrb_match_q(2); 
lrb_match(10) <= evn_seq; 
-- if NoMiss set, do not insert HTR event unless timeout
lrb_match(9) <= NoMiss; 
-- if EnHTRtimeout bit is not set, DCC waits for HTR event forever
--lrb_match(8) <= lrb_match_q(1);
-- lrb_evn(41) and evn_out(68) are calibration trigger flags 
lrb_match(8) <= lrb_evn(41) xor evn_out(68); 
lrb_match(7) <= htr_buf_cnt_r(0) and not two_htr_buf;  
-- when only one HTR event, bit 6 and 5 can be used to input extra info
lrb_match(6) <= two_htr_buf when lrb_match(7) = '0' else lrb_match_q(1); 
--lrb_match(5) <= matching(2) when lrb_match(7) = '0' else lrb_match_q(0); 
lrb_match(5) <= matching(2); 
lrb_match(4) <= one_evn; 
lrb_match(3) <= '1' when evn_diff(4 downto 0) = "11111" or evn_diff(4 downto 0) = "00001" or evn_diff = x"000000" else '0'; 
lrb_match(2) <= '1' when evn_diff(23 downto 4) = x"fffff" or evn_diff(3 downto 0) = "0000" else '0'; 
lrb_match(1) <= '1' when evn_diff(23 downto 4) = x"00000" or evn_diff(3 downto 0) = "0000" else '0'; 
lrb_match(0) <= htr_time_out when matching(2) = '0' else lrb_match_q(0);  
-- if NoSkip bit timeout(19) is set, convert skip to start_read
start_read <= matching(3) or (matching(0) and NoSkip); 
check_next <= matching(2); 
missing <= matching(1); 
-- if NoSkip bit timeout(19) is set, convert skip to start_read
skip <= matching(0) and not NoSkip; 
i_evn_delta: adsuN
   generic map( N => 8)
   port map(
         a => lrb_evn(7 downto 0),
         b => last_evn,
         ci => '1',
         add => '0',
         s => evn_delta);
evn_seq <= '1' when evn_delta = x"01" else '0';
i_lrb_match_q: ram16XNs
    generic map( N => 3)
    port map(
        clk => clk,
        we => we_last_evn, 
        d => lrb_match(3 downto 1),
        a => rp_addr,
        o => lrb_match_q);			
i_last_evn: ram16XNs
    generic map( N => 8)
    port map(
        clk => clk,
        we => we_last_evn, 
        d => lrb_evn(7 downto 0),
        a => rp_addr,
        o => last_evn);			
we_last_evn <= '1' when strobe = '1' and htr_scan = '1' and (new_rp_dl = '1' or (two_htr_buf = '0' and htr_buf_cnt_r(0) = '1')) else '0';
-- htr assembling
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   rst_done <= '0';
	   en_scan <= '0';
	   htr_scan <= '0';
	 elsif(strobe = '1')then
	   rst_done <= rst_cycle_dl and not rst_cycle;
	   en_scan <= ec_header_fifo_wa_h or (en_scan and not htr_scan) or rst_done;
	   if(evn_avl = '1' and en_scan = '1' and full_header_fifo = '0' and htr_req_que_wc(8) = '0')then
		  htr_scan <= '1';
		elsif(stop_htr_scan = '1')then
		  htr_scan <= '0';
		end if;
    end if;
  end if;
end process;
stop_htr_scan <= ((wr_htr_req_que and last_htr_blk) or fake_lrb or missing) and last_lrb;
process(clk)
begin
  if(clk'event and clk = '1')then
    lrb_wa(10 downto 4) <= wp;
  end if;
end process;
lrb_wa(3 downto 0) <= wp_addr;
evn_ra(10 downto 4) <= rp when two_htr_buf = '0' or new_rp = '1' or en_scan = '1' else rp_plus_one;
evn_ra(3 downto 0) <= rp_addr;
smry_ra <= rp & rp_addr;
I_lrb_info: lrb_info PORT MAP(
		reset => reset,
		clk => clk,
		strobe => strobe,
		rd_status_cntr => rd_status_cntr,
		addr => addr,
		r_data => r_data,
		wr_stateLUT => wr_stateLUT,
		rd_stateLUT => rd_stateLUT,
		w_data => w_data,
		HTR_ow => HTR_ow,
		HTR_bz => HTR_bz,
		HTR_state => HTR_state,
		src_sel => src_sel(0),
		lrb1_we => lrb1_we,
		lrb2_we => lrb2_we,
		lrb_data => lrb_data_i,
		lrb1_data => lrb1_data,
		lrb2_data => lrb2_data,
		lrb_wa => lrb_wa,
		evn_ra => evn_ra,
		smry_ra => smry_ra,
		lrb_evn => lrb_evn,
		summary => summary,
		lrb_total_wc => lrb_total_wc,
		wr_lrb_total_wc => wr_lrb_total_wc
	);
I_rd_lrb_wc: rd_lrb_wc PORT MAP(
		reset => reset,
		strobe => strobe,
		clk => clk,
		scan_cntr_que_a => scan_cntr_que_a,
		lrb_total_wc => lrb_total_wc,
		wp_addr => wp_addr,
		wr_lrb_total_wc => wr_lrb_total_wc,
		rd_cntr_que => rd_cntr_que_c,
		cntr_que => cntr_que_c,
		empty_cntr_que => empty_cntr_que_c
	);
------------------------------------------
-- source id and time_out
i_to_cnt: ram16XNs
    generic map( N => 2)
    port map(
        clk => clk,
        we => ec_rp_addr, 
        d => d_to_cnt,
        a => rp_addr,
        o => to_cnt);
d_to_cnt(1) <= not rst_cycle and ((to_cnt(0) and (missing xor to_cnt(1))) or (missing and to_cnt(1)));
d_to_cnt(0) <= not rst_cycle and ((not to_cnt(0) and (missing xor to_cnt(1))) or (missing and to_cnt(1)));
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
-- three consecutive timeout then wait no more
      htr_time_out <= ((to_cnt(1) and to_cnt(0)) or htr_wait_ovfl) and not(wr_htr_req_que or fake_lrb or missing);
      ld_evn_wait <= not en_scan or full_header_fifo or not(two_htr_buf or htr_buf_cnt_r(0));
      ld_htr_wait <= wr_htr_req_que or fake_lrb or missing or not htr_scan;
	 end if;
  end if;
end process;
d_evn_wait <= (not timeout(15 downto 8)) & "00";
i_evn_wait_cntr: l_cntr
  generic map(N => 10)
  port map(
    d => d_evn_wait,
    load => ld_evn_wait,
    reset => '0',
    set => '0',
    ec => strobe,
-- this is L1ac timeout enable bit
    en_cnt => EnFakeL1ac,
    clk => clk,
	 carry => evn_wait_ovfl);
d_htr_wait <= (not timeout(7 downto 0)) & "0000000";
i_htr_wait_cntr: l_cntr
  generic map(N => 15)
  port map(
    d => d_htr_wait,
    load => ld_htr_wait,
    reset => '0',
    set => '0',
    ec => strobe,
-- this is HTR timeout enable bit
    en_cnt => EnHTRtimeout,
    clk => clk,
	 carry => htr_wait_ovfl);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
		if(wr_timeout = '1')then
        timeout(21 downto 0) <= w_data(21 downto 0);
		end if;
		if(wr_src_id = '1')then
-- bits 23-20 evt_ty, bits 19-12 evt_stat, bits 11-0 src_id
        source_id(23 downto 0) <= w_data(23 downto 0);
		end if;
	 end if;
  end if;
end process;
NoL1ac <= timeout(21);
NoWaitNext <= timeout(20);
NoSkip <= timeout(19);
NoMiss <= timeout(18);
EnFakeL1ac <= timeout(17);
EnHTRtimeout <= timeout(16);
timeout(31 downto 22) <= (others => '0');
source_id(31 downto 24) <= (others => '0');
r_data <= timeout when rd_timeout = '1' else (others => 'Z');
r_data <= source_id when rd_src_id = '1' else (others => 'Z');
r_data <= counters when rd_counters = '1' else (others => 'Z');
r_data <= evn_mm_buf when rd_evn_mm = '1' else (others => 'Z');
counters <= counter_rb(31 downto 0) when addr(0) = '0' else 
            counter_high when addr(6 downto 1) = old_cntr_addr(6 downto 1) and old_cntr_addr(0) = '0' else
				counter_rb(63 downto 32);
-- DCC error history
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset_err = '1')then
	   dcc_err <= (others => '0');
    elsif(strobe = '1')then
	   dcc_err(0) <= dcc_err(0) or TTS(0);
	   dcc_err(1) <= dcc_err(1) or TTS(2);
	   dcc_err(2) <= dcc_err(2) or TTS(1);
	   dcc_err(3) <= dcc_err(3) or L1_EvN_mm;
	   dcc_err(4) <= dcc_err(4) or L1_BcN_mm;
	   dcc_err(5) <= dcc_err(5) or CT_EvN_mm;
	   dcc_err(6) <= dcc_err(6) or CT_BcN_mm;
	   dcc_err(7) <= dcc_err(7) or inc_bcnterr;
	   dcc_err(8) <= dcc_err(8) or inc_serr;
	   dcc_err(9) <= dcc_err(9) or inc_derr;
	 end if;
  end if;
end process;
-- 32 bit counters
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   evt_bldr_bc <= "00";
    elsif(strobe = '1')then
	   evt_bldr_bc(1) <= not ld_evt_bldr_bc and (evt_bldr_bc(1) xor (evt_bldr_bc(0) and wr_tail_dl));
	   evt_bldr_bc(0) <= (not ld_evt_bldr_bc and evt_bldr_bc(0)) xor wr_tail_dl;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   slink_bc <= "00";
    elsif(strobe = '1')then
	   slink_bc(1) <= not ld_slink_bc and (slink_bc(1) xor (slink_bc(0) and inc_slink_bc));
	   slink_bc(0) <= (not ld_slink_bc and slink_bc(0)) xor inc_slink_bc;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   mon_cntr <= "00";
    elsif(strobe = '1')then
	   mon_cntr(1) <= not ld_mon_cntr and (mon_cntr(1) xor (mon_cntr(0) and inc_mon_cntr));
	   mon_cntr(0) <= (not ld_mon_cntr and mon_cntr(0)) xor inc_mon_cntr;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   rate_OFW_cntr <= "00";
    elsif(strobe = '1')then
	   rate_OFW_cntr(1) <= not ld_rate_OFW_cntr and (rate_OFW_cntr(1) xor (rate_OFW_cntr(0) and inc_rate_OFW));
	   rate_OFW_cntr(0) <= (not ld_rate_OFW_cntr and rate_OFW_cntr(0)) xor inc_rate_OFW;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   l1ac_cntr <= "00";
    elsif(strobe = '1')then
	   l1ac_cntr(1) <= not ld_l1ac_cntr and (l1ac_cntr(1) xor (l1ac_cntr(0) and inc_l1ac));
	   l1ac_cntr(0) <= (not ld_l1ac_cntr and l1ac_cntr(0)) xor inc_l1ac;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   CT_EvN_mmc <= "00";
    elsif(strobe = '1')then
	   CT_EvN_mmc(1) <= not ld_CT_EvN_mmc and (CT_EvN_mmc(1) xor (CT_EvN_mmc(0) and CT_EvN_mm));
	   CT_EvN_mmc(0) <= (not ld_CT_EvN_mmc and CT_EvN_mmc(0)) xor CT_EvN_mm;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   CT_BcN_mmc <= "00";
    elsif(strobe = '1')then
	   CT_BcN_mmc(1) <= not ld_CT_BcN_mmc and (CT_BcN_mmc(1) xor (CT_BcN_mmc(0) and CT_BcN_mm));
	   CT_BcN_mmc(0) <= (not ld_CT_BcN_mmc and CT_BcN_mmc(0)) xor CT_BcN_mm;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   L1_BcN_mmc <= "00";
    elsif(strobe = '1')then
	   L1_BcN_mmc(1) <= not ld_L1_BcN_mmc and (L1_BcN_mmc(1) xor (L1_BcN_mmc(0) and L1_BcN_mm));
	   L1_BcN_mmc(0) <= (not ld_L1_BcN_mmc and L1_BcN_mmc(0)) xor L1_BcN_mm;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   L1_EvN_mmc <= "00";
    elsif(strobe = '1')then
	   L1_EvN_mmc(1) <= not ld_L1_EvN_mmc and (L1_EvN_mmc(1) xor (L1_EvN_mmc(0) and L1_EvN_mm));
	   L1_EvN_mmc(0) <= (not ld_L1_EvN_mmc and L1_EvN_mmc(0)) xor L1_EvN_mm;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   bcnterr_cntr <= "00";
    elsif(strobe = '1')then
	   bcnterr_cntr(1) <= not ld_bcnterr_cntr and (bcnterr_cntr(1) xor (bcnterr_cntr(0) and inc_bcnterr));
	   bcnterr_cntr(0) <= (not ld_bcnterr_cntr and bcnterr_cntr(0)) xor inc_bcnterr;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   scan_cntr_b <= (others => '0');
		rst_counterb <= '1';
    elsif(strobe = '1')then
	   scan_cntr_b <= scan_cntr_b + 1;
		if(counterb_addr = x"f")then
		  rst_counterb <= '0';
		end if;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   ld_evt_bldr_bc <= '0';
	   ld_slink_bc <= '0';
	   ld_mon_cntr <= '0';
	   ld_l1ac_cntr <= '0';
	   ld_CT_cntr <= '0';
	   ld_CT_EvN_mmc <= '0';
	   ld_CT_BcN_mmc <= '0';
	   ld_L1_EvN_mmc <= '0';
	   ld_L1_BcN_mmc <= '0';
	   ld_bcnterr_cntr <= '0';
	   ld_rate_OFW_cntr <= '0';
    elsif(strobe = '1')then
	   if(scan_cntr_b = "1111")then
		  ld_evt_bldr_bc <= '1';
		else
		  ld_evt_bldr_bc <= '0';
		end if;
	   if(scan_cntr_b = "0000")then
		  ld_slink_bc <= '1';
		else
		  ld_slink_bc <= '0';
		end if;
	   if(scan_cntr_b = "0001")then
		  ld_mon_cntr <= '1';
		else
		  ld_mon_cntr <= '0';
		end if;
	   if(scan_cntr_b = "0010")then
		  ld_l1ac_cntr <= '1';
		else
		  ld_l1ac_cntr <= '0';
		end if;
	   if(scan_cntr_b = "0011")then
		  ld_CT_cntr <= '1';
		else
		  ld_CT_cntr <= '0';
		end if;
	   if(scan_cntr_b = "0100")then
		  ld_CT_EvN_mmc <= '1';
		else
		  ld_CT_EvN_mmc <= '0';
		end if;
	   if(scan_cntr_b = "0101")then
		  ld_CT_BcN_mmc <= '1';
		else
		  ld_CT_BcN_mmc <= '0';
		end if;
	   if(scan_cntr_b = "0110")then
		  ld_L1_EvN_mmc <= '1';
		else
		  ld_L1_EvN_mmc <= '0';
		end if;
	   if(scan_cntr_b = "0111")then
		  ld_L1_BcN_mmc <= '1';
		else
		  ld_L1_BcN_mmc <= '0';
		end if;
	   if(scan_cntr_b = "1000")then
		  ld_bcnterr_cntr <= '1';
		else
		  ld_bcnterr_cntr <= '0';
		end if;
	   if(scan_cntr_b = "1001")then
		  ld_rate_OFW_cntr <= '1';
		else
		  ld_rate_OFW_cntr <= '0';
		end if;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
	   case scan_cntr_b is
		  when "0000" => counterb_inc(1 downto 0) <= evt_bldr_bc;
		  when "0001" => counterb_inc(1 downto 0) <= slink_bc;
		  when "0010" => counterb_inc(1 downto 0) <= mon_cntr;
		  when "0011" => counterb_inc(1 downto 0) <= l1ac_cntr;
		  when "0100" => counterb_inc(1 downto 0) <= CT_cntr;
		  when "0101" => counterb_inc(1 downto 0) <= CT_EvN_mmc;
		  when "0110" => counterb_inc(1 downto 0) <= CT_BcN_mmc;
		  when "0111" => counterb_inc(1 downto 0) <= L1_EvN_mmc;
		  when "1000" => counterb_inc(1 downto 0) <= L1_BcN_mmc;
		  when "1001" => counterb_inc(1 downto 0) <= bcnterr_cntr;
		  when "1010" => counterb_inc(1 downto 0) <= rate_OFW_cntr;
		  when others => counterb_inc(1 downto 0) <= "00";
		end case;
		counterb_addr <= scan_cntr_b;
		counterb_q <= counterb;
	 end if;
  end if;
end process;
i_counterb: ram16XNd
    generic map( N => 32)
    port map(
        clk => clk,
        we => strobe, 
        d => counterb_in,
        a => counterb_addr,
        dpra => addr(3 downto 0),
        spo => counterb_out,
        dpo => counterb);			
counterb_in <= counterb_inc when rst_counterb = '1' else counterb_out + counterb_inc;
counterb_inc(31 downto 2) <= (others => '0');
r_data <= counterb_q when rd_status_cntr = '1' and addr(7 downto 4) = x"f" else (others => 'Z');
-- 64 bit counters
i_evt_bldr_wc: l_cntr
  generic map(N => 8)
  port map(
    d => d_evt_bldr_wc,
    load => ld_evt_bldr_wc,
    reset => reset,
    set => '0',
    ec => ec_evt_bldr_wc,
    en_cnt => '1',
    clk => clk,
	 q => evt_bldr_wc);
d_evt_bldr_wc <= "0000000" & slink_we_i;
ec_evt_bldr_wc <= ld_evt_bldr_wc or slink_we_i;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_slink_wc = '1')then
	   slink_wc <= (others => '0');
    elsif(strobe = '1')then
	   slink_wc <= slink_wc + inc_slink_wc;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_serr_cntr = '1')then
	   serr_cntr <= (others => '0');
    elsif(strobe = '1')then
	   serr_cntr <= serr_cntr + inc_serr;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_derr_cntr = '1')then
	   derr_cntr <= (others => '0');
    elsif(strobe = '1')then
	   derr_cntr <= derr_cntr + inc_derr;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_ready_cntr = '1')then
	   ready_cntr <= (others => '0');
    elsif(strobe = '1' and TTS(3 downto 2) = "10")then
	   ready_cntr <= ready_cntr + 1;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_busy_cntr = '1')then
	   busy_cntr <= (others => '0');
    elsif(strobe = '1' and TTS(3 downto 2) = "01")then
	   busy_cntr <= busy_cntr + 1;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_ovfl_cntr = '1')then
	   ovfl_cntr <= (others => '0');
    elsif(strobe = '1' and TTS(0) = '1' and TTS(3) = '0')then
	   ovfl_cntr <= ovfl_cntr + 1;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_sync_cntr = '1')then
	   sync_cntr <= (others => '0');
    elsif(strobe = '1' and TTS(1) = '1' and TTS(3) = '0')then
	   sync_cntr <= sync_cntr + 1;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_run_cntr = '1')then
	   run_cntr <= (others => '0');
    elsif(strobe = '1' and run = '1')then
	   run_cntr <= run_cntr + 1;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   scan_cntr <= "0100";
    elsif(strobe = '1' and en_wr_cntr_que_a = '1')then
	   if(scan_cntr = "1100")then
		  scan_cntr <= "0100";
		else
		  scan_cntr <= scan_cntr + 1;
		end if;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
      if(scan_cntr(1) = '1')then
		  mux4_a <= mux2_a;
		  mux4_b <= mux2_c;
		else
		  mux4_a <= mux2_b;
		  mux4_b <= mux2_d;
		end if;
      if(scan_cntr(2) = '1')then
		  cntr_que_a_in(7 downto 0) <= mux4_a;
		else
		  cntr_que_a_in(7 downto 0) <= mux4_b;
		end if;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(scan_cntr(0) = '0')then
	   mux2_a <= "00" & serr_cntr;
		if(scan_cntr(3) = '1')then
		  mux2_b <= '0' & run_cntr;
		else
	     mux2_b <= evt_bldr_wc;
		end if;
	   mux2_c <= '0' & ovfl_cntr;
	   mux2_d <= '0' & ready_cntr;
    else
	   mux2_a <= "00" & derr_cntr;
	   mux2_b <= '0' & slink_wc;
	   mux2_c <= '0' & sync_cntr;
	   mux2_d <= '0' & busy_cntr;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   ld_evt_bldr_wc <= '1';
	   ld_slink_wc <= '1';
	   ld_serr_cntr <= '1';
	   ld_derr_cntr <= '1';
	   ld_ready_cntr <= '1';
	   ld_busy_cntr <= '1';
	   ld_ovfl_cntr <= '1';
	   ld_sync_cntr <= '1';
	   ld_run_cntr <= '1';
    else
	   case scan_cntr is
		  when "0100" => ld_evt_bldr_wc <= scan_cntr_que_a;
		  when "0101" => ld_slink_wc <= scan_cntr_que_a;
		  when "0110" => ld_serr_cntr <= scan_cntr_que_a;
		  when "0111" => ld_derr_cntr <= scan_cntr_que_a;
		  when "1000" => ld_ready_cntr <= scan_cntr_que_a;
		  when "1001" => ld_busy_cntr <= scan_cntr_que_a;
		  when "1010" => ld_ovfl_cntr <= scan_cntr_que_a;
		  when "1011" => ld_sync_cntr <= scan_cntr_que_a;
		  when "1100" => ld_run_cntr <= scan_cntr_que_a;
		  when others => null;
		end case;
	 end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   scan_cntr_que_a_dl <= '0';
	   scan_cntr_que_a_dl2 <= '0';
	   en_wr_cntr_que_a <= '0';
	 elsif(strobe = '1')then
	   scan_cntr_que_a_dl <= scan_cntr_que_a;
	   scan_cntr_que_a_dl2 <= scan_cntr_que_a_dl;
	   en_wr_cntr_que_a <= scan_cntr_que_a_dl2;
	 end if;
  end if;
end process;
wr_cntr_que_a <= '0' when cntr_que_a_in(7 downto 0) = "0000000" or strobe = '0' or
                 en_wr_cntr_que_a = '0' else '1';
cntr_que_a_in(11 downto 8) <= scan_cntr(3 downto 0);
i_cntr_que_a : srl_fiforeg
generic map (N => 12)
  port map(
	din => cntr_que_a_in,
	write => wr_cntr_que_a,
	read => rd_cntr_que_a, 
	clk => clk,
	rst => reset,
	dout => cntr_que_a,
	empty => empty_cntr_que_a);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
	   if(htr_scan = '1' and ((start_read = '1' and bc_evn_match(2 downto 0) /= "111") or
		                        missing = '1' or skip = '1'))then
		  wr_cntr_que_bp <= '1';
		else
		  wr_cntr_que_bp <= '0';
		end if;
		if(missing = '1' or skip = '1')then
		  cntr_que_b_in <= "1000" & missing & rp_addr;
		else
		  cntr_que_b_in <= '0' & not bc_evn_match(2 downto 0) & bc_evn_match(3) & rp_addr;
		end if;
	 end if;
  end if;
end process;
--wr_cntr_que_b <= htr_scan and strobe and ((start_read and not(bc_evn_match(1) and bc_evn_match(0))) or missing);
--wr_cntr_que_b <= strobe when htr_scan = '1' and ((start_read = '1' and bc_evn_match(2 downto 0) /= "111") or missing = '1' or skip = '1');
--cntr_que_b_in <= bc_evn_match(2 downto 0) & bc_evn_match(3) & rp_addr;
wr_cntr_que_b <= wr_cntr_que_bp and strobe;
i_cntr_que_b : srl_fiforeg
generic map (N => 9)
  port map(
--	din => rp_addr,
	din => cntr_que_b_in,
	write => wr_cntr_que_b,
	read => rd_cntr_que_b, 
	clk => clk,
	rst => reset,
	dout => cntr_que_b,
	empty => empty_cntr_que_b);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
		counter_cycle <= "00";
		rd_cntr_que_a <= '0';
		rd_cntr_que_b <= '0';
		rd_cntr_que_c <= '0';
    else
	   counter_cycle(1) <= counter_cycle(0);
		if((empty_cntr_que_b = '0' or empty_cntr_que_a = '0' or empty_cntr_que_c = '0') and counter_cycle = "00" and counter_wap(6) = '1')then
	     counter_cycle(0) <= '1';
		else
	     counter_cycle(0) <= '0';
		end if;
      rd_cntr_que_a <= counter_cycle(0) and sel_cntr_que_a;
      rd_cntr_que_b <= counter_cycle(0) and sel_cntr_que_b;
      rd_cntr_que_c <= counter_cycle(0) and sel_cntr_que_c;
	 end if;
	 wr_counter <= counter_cycle(1) or not counter_wap(6);
	 rst_counter_n <= counter_wap(6);
	 if(counter_cycle = "00")then
	   sel_cntr_que_b <= not empty_cntr_que_b;
	   sel_cntr_que_a <= not empty_cntr_que_a and empty_cntr_que_b;
	   sel_cntr_que_c <= not empty_cntr_que_c and empty_cntr_que_a and empty_cntr_que_b;
	   if(empty_cntr_que_b = '0')then
        increment(10 downto 0) <= "00000000001";
	   elsif(empty_cntr_que_a = '0')then
        increment(10 downto 0) <= "000" & cntr_que_a(7 downto 0);
	   else
        increment(10 downto 0) <= cntr_que_c(10 downto 0);
	   end if;
	 end if;
  end if;
end process;
i_counter_h: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => counter_in(63 downto 32),
  			  DIPA => x"0",
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => en_counter_out,
  			  WEA => wr_counter,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => counter_wa,
			  ADDRB => counter_ra,
			  DOA => counter_out(63 downto 32),
			  DOB => counter_rb(63 downto 32)
			  );
i_counter_l: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => counter_in(31 downto 0),
  			  DIPA => x"0",
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => en_counter_out,
  			  WEA => wr_counter,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => counter_wa,
			  ADDRB => counter_ra,
			  DOA => counter_out(31 downto 0),
			  DOB => counter_rb(31 downto 0)
			  );
-- to avoid write/read conflict
en_counter_out <= not wr_counter;
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(strobe = '1')then
	   if(rd_counters = '1')then
		  counter_high <= counter_rb(63 downto 32);
		  old_cntr_addr <= addr(6 downto 0);
		end if;
	 end if;
  end if;
end process;
-- after reset counter_wap increments until counter_wap(6) is '1'
-- during this process counters are all reset to zero
-- counter_wap(6) stays '1' and it becomes a register
i_counter_wa: l_cntr
  generic map(N => 7)
  port map(
    d => d_counter_wap,
    load => counter_wap(6),
    reset => reset,
    set => '0',
    ec => ec_counter_wap,
    en_cnt => '1',
    clk => clk,
	 q => counter_wap);
ec_counter_wap <= '1' when counter_cycle = "00" else '0';
d_counter_wap <= "100" & cntr_que_b(3 downto 0) when empty_cntr_que_b = '0' else
                 "111" & cntr_que_a(11 downto 8) when empty_cntr_que_a = '0' else
					  '1' & cntr_que_c(16 downto 11);
counter_wa <= "000" & counter_wap(5 downto 0);
counter_ra <= "000" & addr(6 downto 1);
i_counter_in: addN
   generic map( N => 64)
   port map(
         a => counter_out,
         b => increment,
			rst_n => rst_counter_n,
         ci => '0',
         s => counter_in);
increment(63 downto 11) <= (others => '0');
-- event number mismatch counters, added 5/13/08
i_evn_mm_pc: RAMB16_S9_S36
	  generic map(WRITE_MODE_A => "READ_FIRST",SIM_COLLISION_CHECK => "NONE")
  port map(DIA => x"00",
  			  DIPA => "0",
  			  DIB => evn_mm_pc_in,
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => '1',
  			  WEA => clr_evn_mm_pc,
  			  WEB => rd_cntr_que_b,
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => evn_mm_pc_aa,
			  ADDRB => evn_mm_pc_ab,
			  DOA => inc_evn_mm_cntr(7 downto 0),
			  DOB => evn_mm_pc_out
			  );
evn_mm_pc_ab <= x"0" & cntr_que_b(4 downto 0);
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(cntr_que_b(5) = '1')then
      evn_mm_pc_in(7 downto 0) <= evn_mm_pc_out(7 downto 0) + 1;
	 else
      evn_mm_pc_in(7 downto 0) <= evn_mm_pc_out(7 downto 0);
	 end if;
	 if(cntr_que_b(6) = '1')then
      evn_mm_pc_in(15 downto 8) <= evn_mm_pc_out(15 downto 8) + 1;
	 else
      evn_mm_pc_in(15 downto 8) <= evn_mm_pc_out(15 downto 8);
	 end if;
	 if(cntr_que_b(7) = '1')then
      evn_mm_pc_in(23 downto 16) <= evn_mm_pc_out(23 downto 16) + 1;
	 else
      evn_mm_pc_in(23 downto 16) <= evn_mm_pc_out(23 downto 16);
	 end if;
	 if(cntr_que_b(8) = '1')then
      evn_mm_pc_in(31 downto 24) <= evn_mm_pc_out(31 downto 24) + 1;
	 else
      evn_mm_pc_in(31 downto 24) <= evn_mm_pc_out(31 downto 24);
	 end if;
	 if(reset = '1')then
	   rst_mm_cntr <= '1';
	 elsif(evn_mm_pc_aa(6 downto 0) = "1111111")then
	   rst_mm_cntr <= '0';
	 end if;
	 if(reset = '1')then
	   evn_mm_pc_aa <= (others => '0');
	 elsif(rst_mm_cntr = '1' or clr_evn_mm_pc = '1')then
	   evn_mm_pc_aa(6 downto 0) <= evn_mm_pc_aa(6 downto 0) + 1;
	 end if;
	 next_evn_mm_pc_aa <= evn_mm_pc_aa(6 downto 1) + 1;
	 if(rst_mm_cntr = '1')then
	   clr_evn_mm_pc <= '1';
	 elsif(next_evn_mm_pc_aa(5 downto 1) = evn_mm_pc_ab(4 downto 0) and counter_cycle(0) = '1' and sel_cntr_que_b = '1')then
	   clr_evn_mm_pc <= '0';
	 else
	   clr_evn_mm_pc <= '1';
	 end if;
  end if;
end process;
i_evn_mm_cntr: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => evn_mm_cntr_in,
  			  DIPA => x"0",
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => '1',
  			  WEA => wr_evn_mm_cntr,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => evn_mm_cntr_aa,
			  ADDRB => evn_mm_cntr_ab,
			  DOB => evn_mm_cntr_out
			  );
evn_mm_cntr_ab <= "00" & evn_mm_pc_aa(1 downto 0) &  evn_mm_pc_aa(6 downto 2);
i_evn_mm_cntr_aa:
	delayt_N generic map( N => 9) port map(clk => clk, d => evn_mm_cntr_ab, ce => '1', q => evn_mm_cntr_aa);
i_wr_evn_mm_cntr:
	delayN generic map(N => 2) port map(d => clr_evn_mm_pc, ce => '1', c => clk, q => wr_evn_mm_cntr);
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(rst_mm_cntr = '1')then
	   evn_mm_cntr_in <= (others => '0');
	 else
	   evn_mm_cntr_in <= evn_mm_cntr_out + inc_evn_mm_cntr;
	 end if;
  end if;
end process;
i_evn_mm_buf: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => evn_mm_cntr_in,
  			  DIPA => x"0",
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => strobe,
  			  WEA => wr_evn_mm_cntr,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => evn_mm_cntr_aa,
			  ADDRB => evn_mm_buf_a,
			  DOB => evn_mm_buf
			  );
evn_mm_buf_a <= "00" & addr(6 downto 0);
-- event number mismatch error handling, added 5/11/06
i_mmLUT: ram16XNd
    generic map( N => 8)
    port map(
        clk => clk,
        we => we_mmLUT, 
        d => w_data(7 downto 0),
        a => addr(3 downto 0),
        dpra => mmLUT_scan,
        spo => mmLUT_spo,
        dpo => mmLUT);			
we_mmLUT <= wr_mmLUT and strobe;
mmLUT_scan(3) <= '0'	;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
	   mmLUT_rb <= mmLUT_spo;
	 end if;
	 if(reset = '1')then
	   bcnt_err <= '0';
	 else
	   bcnt_err <= inc_bcnterr or (bcnt_err and not inc_oc);
	 end if;
	 if(reset = '1')then
	   bcnt_err_cntr <= (others => '0');
	 elsif(inc_oc = '1')then
	   if(bcnt_err = '1' and bcnt_err_cntr /= x"f")then
		  bcnt_err_cntr <= bcnt_err_cntr + 1;
		elsif(bcnt_err = '0' and bcnt_err_cntr /= x"0")then
		  bcnt_err_cntr <= bcnt_err_cntr - 1;
		end if;
	 end if;
	 if(reset = '1')then
	   CT_err_cntr <= (others => '0');
	 elsif(ec_header_fifo_wa_h = '1' and evn_out(68) = '1')then
	   if(EvN_mm = '1' and CT_err_cntr(3 downto 0) /= x"f")then
		  CT_err_cntr(3 downto 0) <= CT_err_cntr(3 downto 0) + 1;
		elsif(EvN_mm = '0' and CT_err_cntr(3 downto 0) /= x"0")then
		  CT_err_cntr(3 downto 0) <= CT_err_cntr(3 downto 0) - 1;
		end if;
	   if(BCN_mm = '1' and CT_err_cntr(7 downto 4) /= x"f")then
		  CT_err_cntr(7 downto 4) <= CT_err_cntr(7 downto 4) + 1;
		elsif(BCN_mm = '0' and CT_err_cntr(3 downto 0) /= x"0")then
		  CT_err_cntr(7 downto 4) <= CT_err_cntr(7 downto 4) - 1;
		end if;
	 end if;
	 if(reset = '1')then
	   L1_err_cntr <= (others => '0');
	 elsif(ec_header_fifo_wa_h = '1' and evn_out(68) = '0')then
	   if(EvN_mm = '1' and L1_err_cntr(3 downto 0) /= x"f")then
		  L1_err_cntr(3 downto 0) <= L1_err_cntr(3 downto 0) + 1;
		elsif(EvN_mm = '0' and CT_err_cntr(3 downto 0) /= x"0")then
		  L1_err_cntr(3 downto 0) <= L1_err_cntr(3 downto 0) - 1;
		end if;
	   if(BCN_mm = '1' and L1_err_cntr(7 downto 4) /= x"f")then
		  L1_err_cntr(7 downto 4) <= L1_err_cntr(7 downto 4) + 1;
		elsif(BCN_mm = '0' and CT_err_cntr(3 downto 0) /= x"0")then
		  L1_err_cntr(7 downto 4) <= L1_err_cntr(7 downto 4) - 1;
		end if;
	 end if;
	 if(ec_header_fifo_wa_h = '1')then
	   mmLUT_scan(2 downto 0) <= "000";
	 elsif(mmLUT_scan(2) = '0')then
	   mmLUT_scan(2 downto 0) <= mmLUT_scan(2 downto 0) + 1;
	 end if;
	 if(mmLUT_scan(2) = '1')then
	   err_cntr <= bcnt_err_cntr;
	 else
	   case mmLUT_scan(1 downto 0) is
	     when "00" => err_cntr <= CT_err_cntr(3 downto 0);
	     when "01" => err_cntr <= CT_err_cntr(7 downto 4);
	     when "10" => err_cntr <= L1_err_cntr(3 downto 0);
	     when "11" => err_cntr <= L1_err_cntr(7 downto 4);
	     when others => null;
	   end case;
	 end if;
    threshold <= mmLUT(7 downto 4);	 
	 stateToGo(1) <= mmLUT(3) and mmLUT(2) and (mmLUT(1) xnor mmLUT(0));
	 stateToGo(0) <= (mmLUT(3) and mmLUT(2) and mmLUT(1) and mmLUT(0)) or
						  (not mmLUT(3) and not mmLUT(2) and mmLUT(1) and not mmLUT(0));
	 if(err_cntr > threshold)then
      ttc_state <= stateToGo;	 
	 else
	   ttc_state <= "00";
	 end if;
	 if(ttc_state(1) = '1' or HTR_state(1) = '1')then
      err_state(1) <= '1';	 
	 elsif(strobe = '1')then
	   err_state(1) <= '0';
	 end if;
	 if((ttc_state(0) = '1' and (ttc_state(1) = '1' or HTR_state(1) = '0')) or
	    (HTR_state(0) = '1' and (HTR_state(1) = '1' or ttc_state(1) = '0')))then
      err_state(0) <= '1';	 
	 elsif(strobe = '1')then
	   err_state(0) <= '0';
	 end if;
  end if;
end process;
r_data <= x"000000" & mmLUT_rb when rd_mmLUT = '1' else (others => 'Z');
faked_evn <= evn_out(71) and evn_out(70) and evn_out(69);
process(addr,evn_out)
begin
  case addr(1 downto 0) is
    when "00" => evn_data <= "000000" & faked_evn & evn_out(68) & evn_out(23 downto 0);
    when "01" => evn_data <= x"00000" & evn_out(35 downto 24);
    when "10" => evn_data <= evn_out(67 downto 36);
    when others => evn_data <= x"00000000";
  end case;
end process;
r_data <= evn_data when rd_evn_buf = '1' else (others => 'Z');
r_data <= "00" & evn_fifo_wa & "000000" & evn_fifo_ra & x"0" when rd_evn_pntr = '1' else (others => 'Z');
-- debugging
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
--	   debug(31 downto 0) <= (others => '0');
	 elsif(strobe = '1')then
--	   debug(15 downto 0) <= debug(15 downto 0) + L1_BcN_mm;
--	   debug(31 downto 16) <= debug(31 downto 16) + ec_header_fifo_wa_h;
	 end if;
    if(strobe = '1')then
	   debug(15 downto 0) <= '0' & dpo_wp_q & '0' & rp;
		debug(31 downto 16) <= matching & lrb_match;
		debug(39 downto 32) <= evn_avl & full_header_fifo & en_scan & htr_scan & rp_addr;
		debug(51 downto 40) <= empty_htr_req_que & full_htr_rd_que & clear & header_fifo_ra;
		debug(55 downto 52) <= slink_full & empty_header_fifo & header_sent & trailer_sent;
		debug(63 downto 56) <= '0' & htr_buf_cnt_r;
		debug(78 downto 64) <= htr_en;
	 end if;
  end if;
end process;
debug(95 downto 79) <= (others => '0');
r_data <= debug(31 downto 0) when rd_debug = '1' and addr(2 downto 0) = "000" else (others => 'Z');
r_data <= debug(63 downto 32) when rd_debug = '1' and addr(2 downto 0) = "001" else (others => 'Z');
--r_data <= debug(95 downto 64) when rd_debug = '1' and addr(1 downto 0) = "11" else (others => 'Z');
end my_arch;
