library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.vtx_pack.all;
use mylib.dcc_pack.all;
entity led_driver is
    Port ( clk : in std_logic;
           strobe : in std_logic;
           err_in : in std_logic_vector(23 downto 0);
           leds : in std_logic_vector(23 downto 0);
-- checking read back data from LED shifter
			  display_err : out std_logic;
           led_pads : inout led_pad);
end led_driver;

architecture my_arch of led_driver is
signal eightMHz : std_logic;
signal hundred_ms : std_logic;
signal h_3cb00 : std_logic_vector(19 downto 0);
signal led_clock : std_logic;
signal blink : std_logic;
signal half_sec : std_logic;
signal five_sec : std_logic;
signal ce_five_sec : std_logic;
signal err_rst : std_logic;
signal sample : std_logic;
signal led_in : std_logic_vector(1 downto 0);
signal display_err_i : std_logic;
signal err_seen: std_logic_vector(23 downto 0);
signal error: std_logic_vector(23 downto 0);
signal latch: std_logic_vector(23 downto 0);
signal pwr_on : std_logic;
signal test : std_logic;
signal tp: std_logic_vector(23 downto 0);
signal sr: std_logic_vector(25 downto 0);
signal sr_old: std_logic_vector(23 downto 0);
signal en_shift : std_logic;
signal start : std_logic;
signal start_dl_a : std_logic;
signal start_dl_b : std_logic;
signal end_shift : std_logic;
signal ec_sr : std_logic;
attribute init: string;
attribute init of pwr_on: signal is "S";
attribute init of test: signal is "S";
begin
--	using lsr as clock divider to generate 8MHz signal
i_eightMHz: delayN generic map(N => 8,INIT => x"0001") port map(d => eightMHz, ce => strobe, c => clk, q => eightMHz);
-- clock divider to get 100ms signal
h_3cb00 <= x"3cb00";
i_hundred_ms: l_cntr
  generic map(N => 20)
  port map(
    d => h_3cb00,
    load => sample,
    reset => '0',
    set => '0',
    ec => strobe,
    en_cnt => eightMHz,
    clk => clk,
	 carry => hundred_ms);
i_half_sec: delayN generic map(N => 10,INIT => x"0003") port map(d => half_sec, ce => sample, c => clk, q => half_sec);
i_five_sec: delayN generic map(N => 10,INIT => x"0001") port map(d => five_sec, ce => ce_five_sec, c => clk, q => five_sec);
ce_five_sec <= strobe and sample and half_sec;
err_rst <= sample and half_sec and five_sec;
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(strobe = '1')then
      if(sample = '1' and half_sec = '1' and blink = '1')then
		  pwr_on <= pwr_on and not test;
		  test <= test and not tp(23);
		  tp <= tp(22 downto 0) & pwr_on;
	   end if;	
-- blibk in one second period to indicate error conditions
      if(sample = '1')then
		  blink <= blink xor half_sec;
	   end if;	
-- LED shift clock is 4MHz according to the datasheets for 74HC595
	   led_clock <= (led_clock xor eightMHz) and en_shift;
		sample <= hundred_ms;
-- for none persisting errors, error blinking mode lasts five to ten senconds
		if(err_rst = '1')then
		  err_seen <= error xor err_seen; 
		end if;
		for i in error'range loop
		  error(i) <= err_in(i) or (error(i) and not (err_rst and err_seen(i)));
		end loop;
		for i in latch'range loop
		  if(test = '1')then
		    latch(i) <= tp(i);
		  elsif(error(i) = '1')then
			 latch(i) <= blink;
		  else
			 latch(i) <= leds(i) or (latch(i) and not sample);
		  end if;
		end loop;
		en_shift <= sample or (en_shift and not end_shift);
		if(sample = '1')then
		  display_err <= display_err_i;
		end if;
		if(sample = '1')then
		  start <= '1';
		  end_shift <= '0';
		  sr <= "00" & latch;
		  sr_old <= sr(23 downto 0);
		  display_err_i <= '0';
		elsif(led_clock = '1' and eightMHz = '1')then
		  start <= '0';
		  end_shift <= start_dl_b;
		  sr <= sr(24 downto 0) & sr(25);
		  sr_old <= sr_old(22 downto 0) & '0';
		  if(sr_old(23) = led_in(1))then
		    display_err_i <= '1';
		  end if;
		end if;
    end if;
	 led_pads.c <= led_clock;
	 if(ec_sr = '1')then
	   led_pads.o <= not sr(23);
		led_in <= led_in(0) & led_pads.i;
	 end if;
  end if;
end process;
ec_sr <= strobe and led_clock and eightMHz;
-- including input and output pads registers, 26 shift clocks are needed
i_start_dl_a: delayN generic map(N => 16) port map(d => start, ce => ec_sr, c => clk, q => start_dl_a);
i_start_dl_b: delayN generic map(N => 9) port map(d => start_dl_a, ce => ec_sr, c => clk, q => start_dl_b);
end my_arch;
