-- This module spies HTR data and save them in the fifo for
-- later retrieval at event building time
library IEEE,mylib;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use mylib.vtx_pack.all;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;

entity lrb_info is
    port (
-- reset input, should be connected to the inverted LOCK output of dll
        reset: in STD_LOGIC;
-- system clock input
        clk: in STD_LOGIC;
        strobe: in STD_LOGIC;
-- read HTR error counter
        rd_status_cntr : in std_logic;
        addr: in STD_LOGIC_VECTOR (13 downto 0);
        r_data: out STD_LOGIC_VECTOR (31 downto 0);
        wr_stateLUT : in std_logic;
        rd_stateLUT : in std_logic;
        w_data: in STD_LOGIC_VECTOR (31 downto 0);
-- HTR states output 
		  HTR_ow : out std_logic;
		  HTR_bz : out std_logic;
        HTR_state : out std_logic_vector(1 downto 0);
-- LRB data spying
-- lrb_we signals lrb_data are valid
		  src_sel : IN std_logic;
		  lrb1_we : IN std_logic;
		  lrb2_we : IN std_logic;
-- bit 34 reset flag
-- bit 33 and 32 used to indicate header and trailer of the block
		  lrb_data : IN std_logic_vector(35 downto 0);
		  lrb1_data : IN std_logic_vector(35 downto 0);
		  lrb2_data : IN std_logic_vector(35 downto 0);
		  lrb_wa : IN std_logic_vector(10 downto 0);
		  evn_ra : IN std_logic_vector(10 downto 0);
		  smry_ra : IN std_logic_vector(10 downto 0);
        lrb_evn : out std_logic_vector(41 downto 0);
        summary : out std_logic_vector(31 downto 0);
        lrb_total_wc : out std_logic_vector(9 downto 0);
		  wr_lrb_total_wc : out std_logic
    );
end lrb_info;

architecture my_arch of lrb_info is
signal lrb_we: std_logic := '0';
signal wr_evn: std_logic := '0';
signal evn_in: std_logic_vector(23 downto 0) := (others => '0');
signal wr_bcn: std_logic := '0';
signal htr_oc_in: std_logic_vector(7 downto 0) := (others => '0');
signal htr_oc: std_logic_vector(7 downto 0) := (others => '0');
signal bcn: std_logic_vector(15 downto 0) := (others => '0');
signal wr_HTRstatus: std_logic := '0';
signal trailer_in: std_logic_vector(23 downto 0) := (others => '0');
signal trailer: std_logic_vector(23 downto 0) := (others => '0');
signal total_wc1: std_logic_vector(9 downto 0) := "0000000000";
signal total_wc2: std_logic_vector(9 downto 0) := "0000000000";
signal total_wc: std_logic_vector(9 downto 0) := "0000000000";
signal htr_ud_cntr_in: std_logic_vector(31 downto 0) := (others => '0');
signal htr_ud_cntr_out: std_logic_vector(31 downto 0) := (others => '0');
signal htr_cntr_in: std_logic_vector(63 downto 0) := (others => '0');
signal htr_cntr_out: std_logic_vector(63 downto 0) := (others => '0');
signal htr_cntr_wa: std_logic_vector(8 downto 0) := (others => '0');
signal htr_cntr_ra: std_logic_vector(8 downto 0) := (others => '0');
signal wr_cntr: std_logic_vector(2 downto 0) := (others => '0');
signal rst_cycle : std_logic := '0';
signal check_HTR_status : std_logic := '0';
signal htr_status_p : std_logic_vector(18 downto 0) := (others => '0');
signal htr_status : std_logic_vector(15 downto 0) := (others => '0');
signal inc_cntr : std_logic_vector(7 downto 0) := (others => '0');
signal htr_buf_in: std_logic_vector(63 downto 0) := (others => '0');
signal htr_buf_out: std_logic_vector(7 downto 0) := (others => '0');
signal htr_buf_ra: std_logic_vector(11 downto 0) := (others => '0');
signal threshold_addra : std_logic_vector(11 downto 0) := (others => '0');
signal threshold_addrb : std_logic_vector(8 downto 0) := (others => '0');
signal stateToGo_addrb : std_logic_vector(8 downto 0) := (others => '0');
signal state_LUT : std_logic_vector(7 downto 0) := (others => '0');
signal threshold : std_logic_vector(31 downto 0) := (others => '0');
signal wr_state : std_logic := '0';
signal state : std_logic_vector(3 downto 0) := (others => '0');
signal stateToGo_in : std_logic_vector(3 downto 0) := (others => '0');
signal stateToGo : std_logic_vector(31 downto 0) := (others => '0');
signal merge_state_addr : std_logic_vector(25 downto 0) := (others => '0');
signal merge_state : std_logic_vector(3 downto 0) := (others => '0');
signal gt_threshold : std_logic_vector(7 downto 0) := (others => '0');
signal merge_ow : std_logic_vector(3 downto 0) := (others => '0');
signal merge_bz : std_logic_vector(3 downto 0) := (others => '0');
signal htr_num : std_logic_vector(3 downto 0) := (others => '0');
signal ow_tmp : std_logic := '0';
signal bz_tmp : std_logic := '0';
signal update_ow : std_logic := '0';
-- following are busy and overflow warning status for 15 HTR channels
signal bz : std_logic_vector(14 downto 0) := (others => '0');
signal ow : std_logic_vector(14 downto 0) := (others => '0');
-------------
begin
g1: for i in 0 to 2 generate
i_evn: RAMB16_S9_S9
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => evn_in(7+i*8 downto i*8),
  			  DIB => x"00",
			  DIPA => "0",
			  DIPB => "0",
  			  ENA => wr_evn,
  			  ENB => strobe,
  			  WEA => lrb_we,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => lrb_wa,
			  ADDRB => evn_ra,
			  DOB => lrb_evn(7+i*8 downto i*8)
			  );
end generate;
evn_in(7 downto 0) <= lrb_data(7 downto 0); 
evn_in(23 downto 8) <= lrb_data(31 downto 16); 
wr_evn <= lrb_data(33);
i_htrerr: RAMB16_S9_S9
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => lrb_data(7 downto 0),
  			  DIB => x"00",
-- lrb_data(14) is CalibTrig flag
			  DIPA => lrb_data(14 downto 14),
			  DIPB => "0",
  			  ENA => wr_HTRstatus,
  			  ENB => strobe,
  			  WEA => '1',
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => lrb_wa,
			  ADDRB => smry_ra,
			  DOB => summary(31 downto 24),
-- summary(15) is CalibTrig flag
			  DOPB => summary(15 downto 15)
			  );
i_oc: RAMB16_S9_S9
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => htr_oc_in,
  			  DIB => x"00",
			  DIPA => "0",
			  DIPB => "0",
  			  ENA => wr_HTRstatus,
  			  ENB => strobe,
  			  WEA => '1',
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => lrb_wa,
			  ADDRB => evn_ra,
			  DOB => htr_oc
			  );
-- lrb_data(14) is CalibTrig flag
htr_oc_in <= "00" & lrb_data(14) & lrb_data(31 downto 27);
g2: for i in 0 to 1 generate
i_bcn: RAMB16_S9_S9
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => lrb_data(7+i*8 downto i*8),
  			  DIB => x"00",
			  DIPA => "0",
			  DIPB => "0",
  			  ENA => wr_bcn,
  			  ENB => strobe,
  			  WEA => lrb_we,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => lrb_wa,
			  ADDRB => evn_ra,
			  DOB => bcn(7+i*8 downto i*8)
			  );
end generate;
lrb_evn(41 downto 24) <= htr_oc(5 downto 0) & bcn(11 downto 0);
g3: for i in 0 to 2 generate
i_trailer: RAMB16_S9_S9
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => trailer_in(7+i*8 downto i*8),
  			  DIB => x"00",
			  DIPA => "0",
			  DIPB => "0",
  			  ENA => lrb_data(32),
  			  ENB => strobe,
  			  WEA => lrb_we,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => lrb_wa,
			  ADDRB => smry_ra,
			  DOB => trailer(7+i*8 downto i*8)
			  );
end generate;
trailer_in(23 downto 19) <= "00000";
trailer_in(18 downto 9) <= total_wc;
trailer_in(8 downto 0) <= lrb_data(35) & lrb_data(23 downto 16);
-- total word count in 32-bit words
summary(9 downto 0) <= trailer(18 downto 9);
-- summary(11) is truncate bit
summary(14 downto 10) <= "100" & trailer(8) & '0';
-- LRB errors
summary(23 downto 16) <= trailer(7 downto 0);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   lrb_we <= '0';
	   wr_HTRstatus <= '0';
	   wr_bcn <= '0';
	 else
	   lrb_we <= lrb1_we or lrb2_we;
	   wr_HTRstatus <= wr_evn and lrb_we;
	   wr_bcn <= wr_HTRstatus;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(lrb1_we = '1' and lrb1_data(33) = '1')then
		total_wc1 <= "0000000010";
	 elsif(lrb1_we = '1')then
		total_wc1 <= total_wc1 + 1;
    end if;
    if(lrb2_we = '1' and lrb2_data(33) = '1')then
		total_wc2 <= "0000000010";
	 elsif(lrb2_we = '1')then
		total_wc2 <= total_wc2 + 1;
    end if;
    if(src_sel = '0')then
      total_wc <= total_wc1;
	 else
      total_wc <= total_wc2;
	 end if;
    wr_lrb_total_wc <= (lrb1_we and lrb1_data(32)) or (lrb2_we and lrb2_data(32));
  end if;
end process;
lrb_total_wc <= total_wc;
-- HTR status counter, modified 5/5/2006
process(clk)
begin
  if(clk'event and clk = '1')then
	 if(wr_HTRstatus = '1')then
	   htr_status_p <= lrb_wa(3 downto 0) & lrb_data(14 downto 0);
	 end if;
	 if(lrb_we = '1' and lrb_data(32) = '1' and wr_HTRstatus = '0' and lrb_data(23 downto 16) = x"00")then
	   check_HTR_status <= '1';
	 else
	   check_HTR_status <= '0';
	 end if;
    if(reset = '1')then
	   htr_cntr_ra(4 downto 1) <= x"0";
--	 elsif(wr_HTRstatus = '1' and lrb_data(32) = '0')then
	 elsif(check_HTR_status = '1')then
	   htr_cntr_ra(4 downto 1) <= htr_status_p(18 downto 15);
	 elsif(rst_cycle = '1' and htr_cntr_ra(0) = '1')then
	   htr_cntr_ra(4 downto 1) <= htr_cntr_ra(4 downto 1) + 1;
	 end if;
--	 htr_cntr_ra(0) <= not htr_cntr_ra(0) and (wr_HTRstatus or rst_cycle);
--	 if(wr_HTRstatus = '1' and lrb_data(32) = '0')then
--	   htr_status <= '0' & lrb_data(14 downto 0);
	 htr_cntr_ra(0) <= not htr_cntr_ra(0) and (check_HTR_status or rst_cycle);
	 if(check_HTR_status = '1')then
	   htr_status <= '0' & htr_status_p(14 downto 0);
	 else
	   htr_status <= htr_status(7 downto 0) & x"00";
	 end if;
    inc_cntr <= htr_status(15 downto 8);
	 if(strobe = '1')then
	   if(reset = '1')then
	     rst_cycle <= '1';
	   elsif(htr_cntr_ra(4 downto 1) = x"f")then
	     rst_cycle <= '0';
		end if;  
	 end if;
--	 wr_cntr <= (wr_cntr(1) or rst_cycle) & wr_cntr(0) & (((wr_HTRstatus and not lrb_data(32)) or htr_cntr_ra(0)) and not rst_cycle);
	 wr_cntr <= (wr_cntr(1) or rst_cycle) & wr_cntr(0) & ((check_HTR_status or htr_cntr_ra(0)) and not rst_cycle);
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    for i in 0 to 7 loop
      if(inc_cntr(i) = '1' and htr_ud_cntr_out(3+i*4 downto i*4) = threshold(3+i*4 downto i*4))then
		  gt_threshold(i) <= '1';
		else
		  gt_threshold(i) <= '0';
		end if;
		stateToGo_addrb <= threshold_addrb;
		if(rst_cycle = '1')then
	     htr_ud_cntr_in(3+i*4 downto i*4) <= x"0";
--      elsif(inc_cntr(i) = '1' and htr_ud_cntr_out(3+i*4 downto i*4) /= x"f")then
      elsif(inc_cntr(i) = '1' and htr_ud_cntr_out(3+i*4 downto i*4) /= threshold(3+i*4 downto i*4))then
	     htr_ud_cntr_in(3+i*4 downto i*4) <= htr_ud_cntr_out(3+i*4 downto i*4) + 1;
      elsif(inc_cntr(i) = '0' and htr_ud_cntr_out(3+i*4 downto i*4) /= x"0")then
	     htr_ud_cntr_in(3+i*4 downto i*4) <= htr_ud_cntr_out(3+i*4 downto i*4) - 1;
	   else
	     htr_ud_cntr_in(3+i*4 downto i*4) <= htr_ud_cntr_out(3+i*4 downto i*4);
		end if;
		if(rst_cycle = '1')then
		  htr_cntr_in(7+i*8 downto i*8) <= x"00";
      elsif(inc_cntr(i) = '1' and htr_cntr_out(7+i*8 downto i*8) /= x"ff")then
	     htr_cntr_in(7+i*8 downto i*8) <= htr_cntr_out(7+i*8 downto i*8) + 1;
	   else
	     htr_cntr_in(7+i*8 downto i*8) <= htr_cntr_out(7+i*8 downto i*8);
	   end if;
	 end loop;	
  end if;
end process;
i_HTRstatus_ud_cntr: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => x"00000000",
  			  DIPA => x"0",
  			  DIB => htr_ud_cntr_in,
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => '1',
  			  WEA => '0',
  			  WEB => wr_cntr(2),
  			  SSRA => rst_cycle,
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => htr_cntr_ra,
			  ADDRB => htr_cntr_wa,
			  DOA => htr_ud_cntr_out
			  );
g4: for i in 0 to 1 generate
  i_HTRstatus_cntr: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => x"00000000",
  			  DIPA => x"0",
  			  DIB => htr_cntr_in(31+32*i downto 32*i),
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => '1',
  			  WEA => '0',
  			  WEB => wr_cntr(2),
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => htr_cntr_ra,
			  ADDRB => htr_cntr_wa,
			  DOA => htr_cntr_out(31+32*i downto 32*i)
			  );
end generate;			  
htr_cntr_ra(8 downto 5) <= "0000";
htr_cntr_wa(8 downto 5) <= "0000";
i_htr_cntr_wa: delayt_N generic map(N => 5) 
          port map(clk => clk, d => htr_cntr_ra(4 downto 0), ce => '1', q => htr_cntr_wa(4 downto 0));
g5: for i in 0 to 7 generate
  htr_buf_in(35+4*i downto 32+4*i) <= htr_cntr_in(7+8*i downto 4+8*i);
  htr_buf_in(3+4*i downto 4*i) <= htr_cntr_in(3+8*i downto 8*i);
end generate;
g6: for i in 0 to 1 generate
i_HTRstatus_buf: RAMB16_S4_S36
  port map(DIA => x"0",
  			  DIB => htr_buf_in(31+32*i downto 32*i),
  			  DIPB => x"0",
  			  ENA => strobe,
  			  ENB => '1',
  			  WEA => '0',
  			  WEB => wr_cntr(2),
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => htr_buf_ra,
			  ADDRB => htr_cntr_wa,
			  DOA => htr_buf_out(3+4*i downto 4*i)
			  );
end generate;
htr_buf_ra <= x"0" & addr(7 downto 0);
r_data <= x"000000" & htr_buf_out when rd_status_cntr = '1' and addr(7 downto 4) /= x"f" else (others => 'Z');
i_threshold: RAMB16_S4_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => w_data(7 downto 4),
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => strobe,
  			  ENB => '1',
  			  WEA => wr_stateLUT,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => threshold_addra,
			  ADDRB => threshold_addrb,
			  DOA => state_LUT(7 downto 4),
			  DOB => threshold
			  );
threshold_addra <= x"00" & addr(3 downto 0);
threshold_addrb(8 downto 1) <= x"00";
threshold_addrb(0) <= htr_cntr_ra(0);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
	   state_LUT(3 downto 0) <= state;
	 end if;
--	 threshold_addrb(0) <= htr_cntr_ra(0);
  end if;
end process;
i_state: ram16XNs
    generic map( N => 4)
    port map(
        clk => clk,
        we => wr_state, 
        d => w_data(3 downto 0),
        a => addr(3 downto 0),
        o => state
        );			
wr_state <= strobe and wr_stateLUT;
r_data <= x"000000" & state_LUT when rd_stateLUT = '1' else (others => 'Z');
-- legal states defined as: "0001" overflow warning, "0010" out of sync, "0100" busy
--                          "1000" ready, "1100" error, "1111" disconnected
-- stateToGo_in(1 downto 0)
-- "01" out of sync
-- "10" error
-- "11" disconnected
i_stateConvert0: ROM16X1
   generic map(INIT => x"8004")
	port map(     
      A0 => w_data(0),
      A1 => w_data(1),
      A2 => w_data(2),
      A3 => w_data(3),
      O  => stateToGo_in(0));
i_stateConvert1: ROM16X1
   generic map(INIT => x"9000")
	port map(     
      A0 => w_data(0),
      A1 => w_data(1),
      A2 => w_data(2),
      A3 => w_data(3),
      O  => stateToGo_in(1));
-- stateToGo_in(2) '1' when overflow warning
i_stateConvert2: ROM16X1
   generic map(INIT => x"0002")
	port map(     
      A0 => w_data(0),
      A1 => w_data(1),
      A2 => w_data(2),
      A3 => w_data(3),
      O  => stateToGo_in(2));
-- stateToGo_in(3)  '1' when busy
i_stateConvert3: ROM16X1
   generic map(INIT => x"0010")
	port map(     
      A0 => w_data(0),
      A1 => w_data(1),
      A2 => w_data(2),
      A3 => w_data(3),
      O  => stateToGo_in(3));
i_stateToGo: RAMB16_S4_S36
  port map(DIA => stateToGo_in,
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => strobe,
  			  ENB => '1',
  			  WEA => wr_stateLUT,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => threshold_addra,
			  ADDRB => stateToGo_addrb,
			  DOB => stateToGo
			  );
i_merge_state: RAMB16_S2_S2
-- INIT_00 to INIt_1F of i_merge_state are all '0'
-- INIT_20 thru INIT_3F must be specified in dcc.ucf file
	  generic map(SIM_COLLISION_CHECK => "NONE",
      INIT_20 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_21 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_22 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_23 => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_24 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_25 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_26 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_27 => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_28 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_29 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_2a => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_2b => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_2c => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_2d => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_2e => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_2f => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_30 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_31 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_32 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_33 => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_34 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e555e555e555e555e555",
      INIT_35 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e555e555e555e555e555",
      INIT_36 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e555e555e555e555e555",
      INIT_37 => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_38 => x"ffffeaaaeaaaeaaaeaaaeaaaeaaaeaaaffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_39 => x"ffffeaaaeaaaeaaaeaaaeaaaeaaaeaaaffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_3a => x"ffffeaaaeaaaeaaaeaaaeaaaeaaaeaaaffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_3b => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_3c => x"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff",
      INIT_3d => x"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff",
      INIT_3e => x"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff",
      INIT_3f => x"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff")
  port map(DIA => "00",
  			  DIB => "00",
  			  ENA => '1',
  			  ENB => '1',
  			  WEA => '0',
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => merge_state_addr(25 downto 13),
			  ADDRB => merge_state_addr(12 downto 0),
			  DOA => merge_state(3 downto 2),
			  DOB => merge_state(1 downto 0)
			  );
--merge_state_addr(12) <= merge_state_addr(25);
process(clk)
begin
  if(clk'event and clk = '1')then
    merge_state_addr(12) <= wr_cntr(2);
    merge_state_addr(25) <= wr_cntr(2);
    for i in 0 to 3 loop
      merge_state_addr(2+i*3 downto i*3) <= gt_threshold(i) & stateToGo(1+i*4 downto i*4);
      merge_state_addr(15+i*3 downto 13+i*3) <= gt_threshold(i+4) & stateToGo(17+i*4 downto 16+i*4);
	 end loop;
  end if;
end process;
--g7: for i in 0 to 7 generate
--  gt_threshold(i) <= '1' when htr_ud_cntr_in(3+i*4 downto i*4) > threshold(3+i*4 downto i*4) else '0';
--end generate;		  
--i_htr_num: delayt_N generic map(N => 4) 
--          port map(clk => clk, d => htr_cntr_ra(4 downto 1), ce => '1', q => htr_num);
i_update_ow: delayN
--  generic map(N=> 5) port map(d => wr_HTRstatus, ce => '1', c => clk, q => update_ow);
  generic map(N=> 5) port map(d => check_HTR_status, ce => '1', c => clk, q => update_ow);
process(clk)
begin
  if(clk'event and clk = '1')then
    for i in 0 to 3 loop
		merge_ow(i) <= (stateToGo(2+i*8) and gt_threshold(i*2)) or (stateToGo(6+i*8) and gt_threshold(i*2+1));  
		merge_bz(i) <= (stateToGo(3+i*8) and gt_threshold(i*2)) or (stateToGo(7+i*8) and gt_threshold(i*2+1));  
	 end loop;
	 if(check_HTR_status = '1')then
	   htr_num <= htr_cntr_ra(4 downto 1);
	 end if;
	 ow_tmp <= merge_ow(0) or merge_ow(1) or merge_ow(2) or merge_ow(3);
	 bz_tmp <= merge_bz(0) or merge_bz(1) or merge_bz(2) or merge_bz(3);
	 if(rst_cycle = '1')then
	   ow <= (others => '0');
	   bz <= (others => '0');
	 elsif(update_ow = '1' and htr_num /= x"f")then
		ow(conv_integer(htr_num)) <= merge_ow(0) or merge_ow(1) or merge_ow(2) or merge_ow(3) or ow_tmp;
		bz(conv_integer(htr_num)) <= merge_bz(0) or merge_bz(1) or merge_bz(2) or merge_bz(3) or bz_tmp;
    end if;
	 if(reset = '1' or rst_cycle = '1')then
	   HTR_ow <= '0';
    elsif(ow /= "000000000000000")then
	   HTR_ow <= '1';
    else
	   HTR_ow <= '0';
    end if;	 
	 if(reset = '1' or rst_cycle = '1')then
	   HTR_bz <= '0';
    elsif(bz /= "000000000000000")then
	   HTR_bz <= '1';
    else
	   HTR_bz <= '0';
    end if;
	 if(reset = '1' or rst_cycle = '1')then
	   HTR_state <= "00";
	 else	
      HTR_state(1) <= merge_state(3) or merge_state(1);	 
      HTR_state(0) <= (merge_state(2) and (merge_state(3) or not merge_state(1))) or
						  (merge_state(0) and (merge_state(1) or not merge_state(3)));
    end if;						  
  end if;
end process;
end my_arch;
