-- This module functions as a front end accumulator of the HTR	word counter
-- word count lrb_total_wc comes together with the HTR number w_addr
-- They are buffered in the fifo i_bc_wc_que to derandomize
-- Then they are added to the accumulator	i_counter which is cleared at reset
-- and every time its content is being transfered periodically to the output fifo
-- i_cntr_que which will be readout by the backend HTR word counter. 
library IEEE,mylib;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use mylib.vtx_pack.all;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;

entity rd_lrb_wc is
    port (
-- reset input, should be connected to the inverted LOCK output of dll
        reset: in STD_LOGIC;
-- system clock input
        clk: in STD_LOGIC;
        strobe: in STD_LOGIC;
        scan_cntr_que_a: out STD_LOGIC;
        lrb_total_wc : in std_logic_vector(9 downto 0);
        wp_addr : in std_logic_vector(3 downto 0);
        wr_lrb_total_wc : in std_logic;
        rd_cntr_que: in STD_LOGIC;
        cntr_que : out std_logic_vector(16 downto 0);
        empty_cntr_que : out std_logic
    );
end rd_lrb_wc;

architecture my_arch of rd_lrb_wc is
signal scan_cntr: std_logic_vector(3 downto 0);
signal bc_wc_que_in: std_logic_vector(13 downto 0);
signal bc_wc_que: std_logic_vector(13 downto 0);
signal rd_bc_wc_que: std_logic;
signal empty_bc_wc_que: std_logic;
signal increment: std_logic_vector(10 downto 0);
signal rst_cycle: std_logic;
signal scan: std_logic;
signal scan_dl: std_logic;
signal tc: std_logic;
signal wr_counter: std_logic;
signal rst_n: std_logic;
signal rst: std_logic;
signal save_and_clr: std_logic;
signal counter_addr: std_logic_vector(3 downto 0);
signal cntr_addr: std_logic_vector(3 downto 0);
signal counter_in: std_logic_vector(18 downto 0);
signal counter_out: std_logic_vector(18 downto 0);
signal htr_info: std_logic_vector(22 downto 0);
signal htr_addr: std_logic_vector(4 downto 0);
signal offset: std_logic_vector(4 downto 0);
signal wr_cntr_que: std_logic;
signal cntr_que_in: std_logic_vector(16 downto 0);
-------------
begin
process(clk)
begin
  if(clk'event and clk = '1') then 
    if(reset = '1')then
	   scan_cntr <= "0000";
	 elsif(strobe = '1')then
	   scan_cntr <= scan_cntr + 1;
	 end if;
    if(scan_cntr(2 downto 0) = "100" and strobe = '0')then
	   scan_cntr_que_a <= '1';
    else
	   scan_cntr_que_a <= '0';
    end if;
  end if;
end process;
bc_wc_que_in <= wp_addr & lrb_total_wc;
i_bc_wc_que : srl_fiforeg
generic map (N => 14)
  port map(
	din => bc_wc_que_in,
	write => wr_lrb_total_wc,
	read => rd_bc_wc_que, 
	clk => clk,
	rst => reset,
	dout => bc_wc_que,
	empty => empty_bc_wc_que);
-- if queue is empty, read has no effect
rd_bc_wc_que <= not save_and_clr;
process(clk)
begin
  if(clk'event and clk = '1') then 
    if(reset = '1')then
	   rst_cycle <= '1'; 
      save_and_clr <= '1';
    else
-- reset all counters
	   rst_cycle <= rst_cycle and not (cntr_addr(3) and cntr_addr(2) and cntr_addr(0));
-- periodic(every 480 clock cycles) transfer and clear local counters
      save_and_clr <= rst_cycle or (scan and strobe); 
    end if;
  end if;
end process;
i_cntr_addr: l_cntr
  generic map(N => 4)
  port map(
    d => x"0",
    load => tc,
    reset => reset,
    set => '0',
    ec => save_and_clr,
    en_cnt => '1',
    clk => clk,
    q => cntr_addr);
tc <= cntr_addr(3) and cntr_addr(2) and cntr_addr(1);
-- LRB word count storage
i_counter: ram16XNs
   generic map (N => 19)
   port map(
        clk => clk,
        we => wr_counter,
        d => counter_in,
        a => counter_addr,
        o => counter_out);
process(clk)
begin
  if(clk'event and clk = '1') then 
    if(save_and_clr = '0')then
-- LRB address
	   counter_addr <= bc_wc_que(13 downto 10); 
    else
-- clear or read out
	   counter_addr <= cntr_addr; 
    end if;
    wr_counter <= save_and_clr or not empty_bc_wc_que;
    increment <= '0' & bc_wc_que(9 downto 0);
	 rst_n <= not save_and_clr;
  end if;
end process;
rst <= not rst_n;
-- block count
i_inc_bc: add_oneN 
generic map (N => 8)
   port map (
        d => counter_out(18 downto 11),
        rst => rst,
        sum => counter_in(18 downto 11));
-- word count
i_adder: addN
   generic map( N => 11)
   port map(
         a => counter_out(10 downto 0),
         b => increment,
			rst_n => rst_n,
         ci => '0',
         s => counter_in(10 downto 0));
process(clk)
begin
  if(clk'event and clk = '1') then 
    if(strobe = '1' and scan_dl = '1')then
	   htr_info <= counter_addr & counter_out;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1') then 
    if(reset = '1')then
		scan <= '0';
		scan_dl <= '0';
		wr_cntr_que <= '0';
    elsif(strobe = '1')then
	   scan <= scan_cntr(3) and scan_cntr(2) and scan_cntr(1) and scan_cntr(0);
		scan_dl <= scan;
		if(scan_dl = '1' and counter_out(18 downto 11) /= x"00")then
-- output if block count non zero
		  wr_cntr_que <= '1';
		else
		  wr_cntr_que <= '0';
		end if;
    end if;
  end if;
end process;
-- this address calculation was left over when maximum HTR per DCC was 18
i_htr_cntr_addr: adsuN
   generic map( N => 5)
   port map(
         a => htr_addr,
         b => offset,
         ci => '0',
         add => '1',
         s => cntr_que_in(16 downto 12));
htr_addr <= "00" & htr_info(22 downto 20);
offset <= strobe & not strobe & '0' & strobe & not strobe;
cntr_que_in(11) <= htr_info(19);
-- multiplexing block count and word count
cntr_que_in(10 downto 0) <= "000" & htr_info(18 downto 11) when strobe = '0' else htr_info(10 downto 0);
i_cntr_que : srl_fiforeg
generic map (N => 17)
  port map(
	din => cntr_que_in,
	write => wr_cntr_que,
	read => rd_cntr_que, 
	clk => clk,
	rst => reset,
	dout => cntr_que,
	empty => empty_cntr_que);
end my_arch;
